# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class DetachMachinePoolFromCloudRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'machine_pool_name': 'str',
        'cloud_id': 'str',
        'cloud_resource_id': 'str'
    }

    attribute_map = {
        'machine_pool_name': 'machine_pool_name',
        'cloud_id': 'cloud_id',
        'cloud_resource_id': 'cloud_resource_id'
    }

    def __init__(self, machine_pool_name=None, cloud_id=None, cloud_resource_id=None, local_vars_configuration=None):  # noqa: E501
        """DetachMachinePoolFromCloudRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._machine_pool_name = None
        self._cloud_id = None
        self._cloud_resource_id = None
        self.discriminator = None

        self.machine_pool_name = machine_pool_name
        self.cloud_id = cloud_id
        if cloud_resource_id is not None:
            self.cloud_resource_id = cloud_resource_id

    @property
    def machine_pool_name(self):
        """Gets the machine_pool_name of this DetachMachinePoolFromCloudRequest.  # noqa: E501

        The name of the machine pool to detach.  # noqa: E501

        :return: The machine_pool_name of this DetachMachinePoolFromCloudRequest.  # noqa: E501
        :rtype: str
        """
        return self._machine_pool_name

    @machine_pool_name.setter
    def machine_pool_name(self, machine_pool_name):
        """Sets the machine_pool_name of this DetachMachinePoolFromCloudRequest.

        The name of the machine pool to detach.  # noqa: E501

        :param machine_pool_name: The machine_pool_name of this DetachMachinePoolFromCloudRequest.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and machine_pool_name is None:  # noqa: E501
            raise ValueError("Invalid value for `machine_pool_name`, must not be `None`")  # noqa: E501

        self._machine_pool_name = machine_pool_name

    @property
    def cloud_id(self):
        """Gets the cloud_id of this DetachMachinePoolFromCloudRequest.  # noqa: E501

        The name of the cloud to detach from this machine pool.  # noqa: E501

        :return: The cloud_id of this DetachMachinePoolFromCloudRequest.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this DetachMachinePoolFromCloudRequest.

        The name of the cloud to detach from this machine pool.  # noqa: E501

        :param cloud_id: The cloud_id of this DetachMachinePoolFromCloudRequest.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    @property
    def cloud_resource_id(self):
        """Gets the cloud_resource_id of this DetachMachinePoolFromCloudRequest.  # noqa: E501

        The ID of the specific cloud resource to detach from. If not specified, detaches from the primary cloud resource in the cloud. Note that this is not the machine pool / PCP cloud resource.  # noqa: E501

        :return: The cloud_resource_id of this DetachMachinePoolFromCloudRequest.  # noqa: E501
        :rtype: str
        """
        return self._cloud_resource_id

    @cloud_resource_id.setter
    def cloud_resource_id(self, cloud_resource_id):
        """Sets the cloud_resource_id of this DetachMachinePoolFromCloudRequest.

        The ID of the specific cloud resource to detach from. If not specified, detaches from the primary cloud resource in the cloud. Note that this is not the machine pool / PCP cloud resource.  # noqa: E501

        :param cloud_resource_id: The cloud_resource_id of this DetachMachinePoolFromCloudRequest.  # noqa: E501
        :type: str
        """

        self._cloud_resource_id = cloud_resource_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DetachMachinePoolFromCloudRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, DetachMachinePoolFromCloudRequest):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class JobEventFields(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'ha_job_id': 'str',
        'error_type': 'HaJobErrorTypes',
        'has_additional_sensitive_message': 'bool',
        'event_type': 'HaJobStates'
    }

    attribute_map = {
        'ha_job_id': 'ha_job_id',
        'error_type': 'error_type',
        'has_additional_sensitive_message': 'has_additional_sensitive_message',
        'event_type': 'event_type'
    }

    def __init__(self, ha_job_id=None, error_type=None, has_additional_sensitive_message=False, event_type=None, local_vars_configuration=None):  # noqa: E501
        """JobEventFields - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._ha_job_id = None
        self._error_type = None
        self._has_additional_sensitive_message = None
        self._event_type = None
        self.discriminator = None

        if ha_job_id is not None:
            self.ha_job_id = ha_job_id
        if error_type is not None:
            self.error_type = error_type
        if has_additional_sensitive_message is not None:
            self.has_additional_sensitive_message = has_additional_sensitive_message
        if event_type is not None:
            self.event_type = event_type

    @property
    def ha_job_id(self):
        """Gets the ha_job_id of this JobEventFields.  # noqa: E501

        The id of the Job.  # noqa: E501

        :return: The ha_job_id of this JobEventFields.  # noqa: E501
        :rtype: str
        """
        return self._ha_job_id

    @ha_job_id.setter
    def ha_job_id(self, ha_job_id):
        """Sets the ha_job_id of this JobEventFields.

        The id of the Job.  # noqa: E501

        :param ha_job_id: The ha_job_id of this JobEventFields.  # noqa: E501
        :type: str
        """

        self._ha_job_id = ha_job_id

    @property
    def error_type(self):
        """Gets the error_type of this JobEventFields.  # noqa: E501

        Type of Job error. Only populated for JOB type events that are in error state  # noqa: E501

        :return: The error_type of this JobEventFields.  # noqa: E501
        :rtype: HaJobErrorTypes
        """
        return self._error_type

    @error_type.setter
    def error_type(self, error_type):
        """Sets the error_type of this JobEventFields.

        Type of Job error. Only populated for JOB type events that are in error state  # noqa: E501

        :param error_type: The error_type of this JobEventFields.  # noqa: E501
        :type: HaJobErrorTypes
        """

        self._error_type = error_type

    @property
    def has_additional_sensitive_message(self):
        """Gets the has_additional_sensitive_message of this JobEventFields.  # noqa: E501

        Whether we have additional sensitive message details stored that can be subsequently fetched.  # noqa: E501

        :return: The has_additional_sensitive_message of this JobEventFields.  # noqa: E501
        :rtype: bool
        """
        return self._has_additional_sensitive_message

    @has_additional_sensitive_message.setter
    def has_additional_sensitive_message(self, has_additional_sensitive_message):
        """Sets the has_additional_sensitive_message of this JobEventFields.

        Whether we have additional sensitive message details stored that can be subsequently fetched.  # noqa: E501

        :param has_additional_sensitive_message: The has_additional_sensitive_message of this JobEventFields.  # noqa: E501
        :type: bool
        """

        self._has_additional_sensitive_message = has_additional_sensitive_message

    @property
    def event_type(self):
        """Gets the event_type of this JobEventFields.  # noqa: E501

        Type of job event  # noqa: E501

        :return: The event_type of this JobEventFields.  # noqa: E501
        :rtype: HaJobStates
        """
        return self._event_type

    @event_type.setter
    def event_type(self, event_type):
        """Sets the event_type of this JobEventFields.

        Type of job event  # noqa: E501

        :param event_type: The event_type of this JobEventFields.  # noqa: E501
        :type: HaJobStates
        """

        self._event_type = event_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, JobEventFields):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, JobEventFields):
            return True

        return self.to_dict() != other.to_dict()



from typing import List, Optional

from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_command
from anyscale.user._private.user_sdk import PrivateUserSDK
from anyscale.user.models import AdminCreatedUser, AdminCreateUser, User


_USER_SDK_SINGLETON_KEY = "user_sdk"

_ADMIN_BATCH_CREATE_EXAMPLE = """
import anyscale
from anyscale.user.models import AdminCreateUser

anyscale.user.admin_batch_create(
    [AdminCreateUser(
        name="name",
        email="test@anyscale.com",
        password="",
        is_sso_user=False,
        lastname="lastname",
        title="title",
    ),],
)
"""

_ADMIN_BATCH_CREATE_ARG_DOCSTRINGS = {
    "admin_create_users": "Users to be created by an admin.",
}


@sdk_command(
    _USER_SDK_SINGLETON_KEY,
    PrivateUserSDK,
    doc_py_example=_ADMIN_BATCH_CREATE_EXAMPLE,
    arg_docstrings=_ADMIN_BATCH_CREATE_ARG_DOCSTRINGS,
)
def admin_batch_create(
    admin_create_users: List[AdminCreateUser],
    *,
    _private_sdk: Optional[PrivateUserSDK] = None,
) -> List[AdminCreatedUser]:
    """Batch create, as an admin, users without email verification.
    """
    return _private_sdk.admin_batch_create(admin_create_users)  # type: ignore


_LIST_EXAMPLE = """
import anyscale

for user in anyscale.user.list(max_items=10):
    print(user.email)
"""

_LIST_ARG_DOCSTRINGS = {
    "email": "Filter collaborators by exact email address.",
    "name": "Filter collaborators by exact display name.",
    "collaborator_type": (
        "Filter by collaborator type. Accepts values such as 'all_accounts', "
        "'only_service_accounts', or 'only_user_accounts'."
    ),
    "is_service_account": "If provided, filter collaborators by whether they are service accounts.",
    "max_items": "Maximum total number of users to yield (default: iterate all).",
    "page_size": "Number of users fetched per API request (default: API default).",
}


@sdk_command(
    _USER_SDK_SINGLETON_KEY,
    PrivateUserSDK,
    doc_py_example=_LIST_EXAMPLE,
    arg_docstrings=_LIST_ARG_DOCSTRINGS,
)
def list(  # noqa: A001
    *,
    email: Optional[str] = None,
    name: Optional[str] = None,
    collaborator_type: Optional[str] = None,
    is_service_account: Optional[bool] = None,
    max_items: Optional[int] = None,
    page_size: Optional[int] = None,
    _private_sdk: Optional[PrivateUserSDK] = None,
) -> ResultIterator[User]:
    """List collaborators within the organization."""
    return _private_sdk.list(  # type: ignore
        email=email,
        name=name,
        collaborator_type=collaborator_type,
        is_service_account=is_service_account,
        max_items=max_items,
        page_size=page_size,
    )


_GET_EXAMPLE = """
import anyscale

user = anyscale.user.get(email="owner@anyscale.com")
print(user.permission_level)
"""

_GET_ARG_DOCSTRINGS = {
    "email": "Email address of the user to retrieve.",
    "name": "Display name of the user to retrieve.",
    "collaborator_type": (
        "Optional collaborator type constraint when fetching the user."
    ),
    "is_service_account": "Filter by whether the user is a service account.",
}


@sdk_command(
    _USER_SDK_SINGLETON_KEY,
    PrivateUserSDK,
    doc_py_example=_GET_EXAMPLE,
    arg_docstrings=_GET_ARG_DOCSTRINGS,
)
def get(
    *,
    email: Optional[str] = None,
    name: Optional[str] = None,
    collaborator_type: Optional[str] = None,
    is_service_account: Optional[bool] = None,
    _private_sdk: Optional[PrivateUserSDK] = None,
) -> User:
    """Retrieve a single collaborator by email or name."""
    return _private_sdk.get(  # type: ignore
        email=email,
        name=name,
        collaborator_type=collaborator_type,
        is_service_account=is_service_account,
    )



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class WorkspaceSystemArtifacts(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'pip_requirements_uri': 'str',
        'skip_packages_tracking_uri': 'str',
        'environment_variables_uri': 'str',
        'dockerfile_uri': 'str',
        'working_dir_content_uri': 'str'
    }

    attribute_map = {
        'pip_requirements_uri': 'pip_requirements_uri',
        'skip_packages_tracking_uri': 'skip_packages_tracking_uri',
        'environment_variables_uri': 'environment_variables_uri',
        'dockerfile_uri': 'dockerfile_uri',
        'working_dir_content_uri': 'working_dir_content_uri'
    }

    def __init__(self, pip_requirements_uri=None, skip_packages_tracking_uri=None, environment_variables_uri=None, dockerfile_uri=None, working_dir_content_uri=None, local_vars_configuration=None):  # noqa: E501
        """WorkspaceSystemArtifacts - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._pip_requirements_uri = None
        self._skip_packages_tracking_uri = None
        self._environment_variables_uri = None
        self._dockerfile_uri = None
        self._working_dir_content_uri = None
        self.discriminator = None

        if pip_requirements_uri is not None:
            self.pip_requirements_uri = pip_requirements_uri
        if skip_packages_tracking_uri is not None:
            self.skip_packages_tracking_uri = skip_packages_tracking_uri
        if environment_variables_uri is not None:
            self.environment_variables_uri = environment_variables_uri
        if dockerfile_uri is not None:
            self.dockerfile_uri = dockerfile_uri
        if working_dir_content_uri is not None:
            self.working_dir_content_uri = working_dir_content_uri

    @property
    def pip_requirements_uri(self):
        """Gets the pip_requirements_uri of this WorkspaceSystemArtifacts.  # noqa: E501

        URI pointing to the pip requirements.txt file.  # noqa: E501

        :return: The pip_requirements_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :rtype: str
        """
        return self._pip_requirements_uri

    @pip_requirements_uri.setter
    def pip_requirements_uri(self, pip_requirements_uri):
        """Sets the pip_requirements_uri of this WorkspaceSystemArtifacts.

        URI pointing to the pip requirements.txt file.  # noqa: E501

        :param pip_requirements_uri: The pip_requirements_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :type: str
        """

        self._pip_requirements_uri = pip_requirements_uri

    @property
    def skip_packages_tracking_uri(self):
        """Gets the skip_packages_tracking_uri of this WorkspaceSystemArtifacts.  # noqa: E501

        URI pointing to the .skip_packages_tracking file.  # noqa: E501

        :return: The skip_packages_tracking_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :rtype: str
        """
        return self._skip_packages_tracking_uri

    @skip_packages_tracking_uri.setter
    def skip_packages_tracking_uri(self, skip_packages_tracking_uri):
        """Sets the skip_packages_tracking_uri of this WorkspaceSystemArtifacts.

        URI pointing to the .skip_packages_tracking file.  # noqa: E501

        :param skip_packages_tracking_uri: The skip_packages_tracking_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :type: str
        """

        self._skip_packages_tracking_uri = skip_packages_tracking_uri

    @property
    def environment_variables_uri(self):
        """Gets the environment_variables_uri of this WorkspaceSystemArtifacts.  # noqa: E501

        URI pointing to the environment variables file.  # noqa: E501

        :return: The environment_variables_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :rtype: str
        """
        return self._environment_variables_uri

    @environment_variables_uri.setter
    def environment_variables_uri(self, environment_variables_uri):
        """Sets the environment_variables_uri of this WorkspaceSystemArtifacts.

        URI pointing to the environment variables file.  # noqa: E501

        :param environment_variables_uri: The environment_variables_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :type: str
        """

        self._environment_variables_uri = environment_variables_uri

    @property
    def dockerfile_uri(self):
        """Gets the dockerfile_uri of this WorkspaceSystemArtifacts.  # noqa: E501

        URI pointing to the Dockerfile (or containerfile).  # noqa: E501

        :return: The dockerfile_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :rtype: str
        """
        return self._dockerfile_uri

    @dockerfile_uri.setter
    def dockerfile_uri(self, dockerfile_uri):
        """Sets the dockerfile_uri of this WorkspaceSystemArtifacts.

        URI pointing to the Dockerfile (or containerfile).  # noqa: E501

        :param dockerfile_uri: The dockerfile_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :type: str
        """

        self._dockerfile_uri = dockerfile_uri

    @property
    def working_dir_content_uri(self):
        """Gets the working_dir_content_uri of this WorkspaceSystemArtifacts.  # noqa: E501

        URI pointing to the working directory content.  # noqa: E501

        :return: The working_dir_content_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :rtype: str
        """
        return self._working_dir_content_uri

    @working_dir_content_uri.setter
    def working_dir_content_uri(self, working_dir_content_uri):
        """Sets the working_dir_content_uri of this WorkspaceSystemArtifacts.

        URI pointing to the working directory content.  # noqa: E501

        :param working_dir_content_uri: The working_dir_content_uri of this WorkspaceSystemArtifacts.  # noqa: E501
        :type: str
        """

        self._working_dir_content_uri = working_dir_content_uri

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorkspaceSystemArtifacts):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, WorkspaceSystemArtifacts):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class AggregatedUsageQuery(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'start_date': 'date',
        'end_date': 'date',
        'name_contains': 'str',
        'group_by_date': 'bool',
        'asc': 'bool'
    }

    attribute_map = {
        'start_date': 'start_date',
        'end_date': 'end_date',
        'name_contains': 'name_contains',
        'group_by_date': 'group_by_date',
        'asc': 'asc'
    }

    def __init__(self, start_date=None, end_date=None, name_contains=None, group_by_date=False, asc=False, local_vars_configuration=None):  # noqa: E501
        """AggregatedUsageQuery - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._start_date = None
        self._end_date = None
        self._name_contains = None
        self._group_by_date = None
        self._asc = None
        self.discriminator = None

        self.start_date = start_date
        self.end_date = end_date
        if name_contains is not None:
            self.name_contains = name_contains
        if group_by_date is not None:
            self.group_by_date = group_by_date
        if asc is not None:
            self.asc = asc

    @property
    def start_date(self):
        """Gets the start_date of this AggregatedUsageQuery.  # noqa: E501

        Start date of the usage data to query  # noqa: E501

        :return: The start_date of this AggregatedUsageQuery.  # noqa: E501
        :rtype: date
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """Sets the start_date of this AggregatedUsageQuery.

        Start date of the usage data to query  # noqa: E501

        :param start_date: The start_date of this AggregatedUsageQuery.  # noqa: E501
        :type: date
        """
        if self.local_vars_configuration.client_side_validation and start_date is None:  # noqa: E501
            raise ValueError("Invalid value for `start_date`, must not be `None`")  # noqa: E501

        self._start_date = start_date

    @property
    def end_date(self):
        """Gets the end_date of this AggregatedUsageQuery.  # noqa: E501

        End date of the usage data to query  # noqa: E501

        :return: The end_date of this AggregatedUsageQuery.  # noqa: E501
        :rtype: date
        """
        return self._end_date

    @end_date.setter
    def end_date(self, end_date):
        """Sets the end_date of this AggregatedUsageQuery.

        End date of the usage data to query  # noqa: E501

        :param end_date: The end_date of this AggregatedUsageQuery.  # noqa: E501
        :type: date
        """
        if self.local_vars_configuration.client_side_validation and end_date is None:  # noqa: E501
            raise ValueError("Invalid value for `end_date`, must not be `None`")  # noqa: E501

        self._end_date = end_date

    @property
    def name_contains(self):
        """Gets the name_contains of this AggregatedUsageQuery.  # noqa: E501

        Filter the usage data by name containing this string  # noqa: E501

        :return: The name_contains of this AggregatedUsageQuery.  # noqa: E501
        :rtype: str
        """
        return self._name_contains

    @name_contains.setter
    def name_contains(self, name_contains):
        """Sets the name_contains of this AggregatedUsageQuery.

        Filter the usage data by name containing this string  # noqa: E501

        :param name_contains: The name_contains of this AggregatedUsageQuery.  # noqa: E501
        :type: str
        """

        self._name_contains = name_contains

    @property
    def group_by_date(self):
        """Gets the group_by_date of this AggregatedUsageQuery.  # noqa: E501

        Whether to group the usage data by date  # noqa: E501

        :return: The group_by_date of this AggregatedUsageQuery.  # noqa: E501
        :rtype: bool
        """
        return self._group_by_date

    @group_by_date.setter
    def group_by_date(self, group_by_date):
        """Sets the group_by_date of this AggregatedUsageQuery.

        Whether to group the usage data by date  # noqa: E501

        :param group_by_date: The group_by_date of this AggregatedUsageQuery.  # noqa: E501
        :type: bool
        """

        self._group_by_date = group_by_date

    @property
    def asc(self):
        """Gets the asc of this AggregatedUsageQuery.  # noqa: E501

        Whether to sort the usage data in ascending order  # noqa: E501

        :return: The asc of this AggregatedUsageQuery.  # noqa: E501
        :rtype: bool
        """
        return self._asc

    @asc.setter
    def asc(self, asc):
        """Sets the asc of this AggregatedUsageQuery.

        Whether to sort the usage data in ascending order  # noqa: E501

        :param asc: The asc of this AggregatedUsageQuery.  # noqa: E501
        :type: bool
        """

        self._asc = asc

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AggregatedUsageQuery):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, AggregatedUsageQuery):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.compute_config._private.compute_config_sdk import PrivateComputeConfigSDK
from anyscale.compute_config.commands import (
    _ARCHIVE_ARG_DOCSTRINGS,
    _ARCHIVE_EXAMPLE,
    _CREATE_ARG_DOCSTRINGS,
    _CREATE_EXAMPLE,
    _GET_ARG_DOCSTRINGS,
    _GET_DEFAULT_ARG_DOCSTRINGS,
    _GET_DEFAULT_EXAMPLE,
    _GET_EXAMPLE,
    _LIST_ARG_DOCSTRINGS,
    _LIST_EXAMPLE,
    archive as archive,
    create as create,
    get as get,
    get_default as get_default,
    list as list,  # noqa: A004
)
from anyscale.compute_config.models import (
    ComputeConfig as ComputeConfig,
    ComputeConfigListResult,
    ComputeConfigType,
    ComputeConfigVersion,
    HeadNodeConfig as HeadNodeConfig,
    MultiResourceComputeConfig as MultiResourceComputeConfig,
    WorkerNodeGroupConfig as WorkerNodeGroupConfig,
)


class ComputeConfigSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateComputeConfigSDK(
            client=client, logger=logger, timer=timer
        )

    @sdk_docs(
        doc_py_example=_CREATE_EXAMPLE, arg_docstrings=_CREATE_ARG_DOCSTRINGS,
    )
    def create(  # noqa: F811
        self, config: ComputeConfigType, *, name: Optional[str],
    ) -> str:
        """Create a new version of a compute config.

        Returns the full name of the registered compute config, including the version.
        """
        full_name, _ = self._private_sdk.create_compute_config(config, name=name)
        return full_name

    @sdk_docs(
        doc_py_example=_GET_EXAMPLE, arg_docstrings=_GET_ARG_DOCSTRINGS,
    )
    def get(  # noqa: F811
        self, name: str, *, include_archived: bool = False, _id: Optional[str] = None,
    ) -> ComputeConfigVersion:
        """Get the compute config with the specified name.

        The name can contain an optional version tag, i.e., 'name:version'.
        If no version is provided, the latest one will be returned.
        """
        # NOTE(edoakes): I want to avoid exposing fetching by ID in the public API,
        # but it's needed for parity with the existing CLI. Therefore I am adding it
        # as a hidden private API that can be used like: (`name="", _id=id`).
        return self._private_sdk.get_compute_config(
            name=name or None, id=_id, include_archived=include_archived
        )

    @sdk_docs(
        doc_py_example=_ARCHIVE_EXAMPLE, arg_docstrings=_ARCHIVE_ARG_DOCSTRINGS,
    )
    def archive(self, name: str, *, _id: Optional[str] = None):  # noqa: F811
        """Archive a compute config and all of its versions.

        The name can contain an optional version, e.g., 'name:version'.
        If no version is provided, the latest one will be archived.

        Once a compute config is archived, its name will no longer be usable in the organization.
        """
        # NOTE(edoakes): I want to avoid exposing fetching by ID in the public API,
        # but it's needed for parity with the existing CLI. Therefore I am adding it
        # as a hidden private API that can be used like: (`name="", _id=id`).
        return self._private_sdk.archive_compute_config(name=name or None, id=_id,)

    @sdk_docs(
        doc_py_example=_GET_DEFAULT_EXAMPLE, arg_docstrings=_GET_DEFAULT_ARG_DOCSTRINGS,
    )
    def get_default(  # noqa: F811
        self, *, cloud: Optional[str] = None, cloud_resource: Optional[str] = None,
    ) -> ComputeConfigVersion:
        """Get the default compute config for the specified cloud.

        Returns the default compute configuration that will be used when no compute config
        is explicitly specified for a workload.

        Args:
            cloud: Name of the cloud. If not provided, uses the organization's default cloud.
            cloud_resource: Name of the cloud resource. If not provided, uses the default
                cloud resource for the cloud.

        Returns:
            ComputeConfigVersion containing the default compute config.
        """
        return self._private_sdk.get_default_compute_config(
            cloud=cloud, cloud_resource=cloud_resource
        )

    @sdk_docs(
        doc_py_example=_LIST_EXAMPLE, arg_docstrings=_LIST_ARG_DOCSTRINGS,
    )
    def list(  # noqa: F811, A003, PLR0913
        self,
        *,
        name: Optional[str] = None,
        cloud_id: Optional[str] = None,
        cloud_name: Optional[str] = None,
        sort_by: str = "last_modified_at",
        sort_order: str = "asc",
        max_items: int = 20,
        next_token: Optional[str] = None,
        include_shared: bool = False,
        _id: Optional[str] = None,
    ) -> ComputeConfigListResult:
        """List compute configurations with filtering, sorting, and pagination.

        Returns a ComputeConfigListResult object containing:
        - results: List of compute config objects
        - next_token: Token for fetching the next page (None if no more results)
        - count: Number of results in this page

        Args:
            name: Filter by compute config name
            cloud_id: Filter by cloud ID
            cloud_name: Filter by cloud name
            sort_by: Field to sort by ('name', 'created_at', 'last_modified_at')
            sort_order: Sort order ('asc' or 'desc')
            max_items: Maximum number of items to return per page
            next_token: Pagination token for next page
            include_shared: Include configs shared with the user
            _id: Internal parameter for fetching by ID

        Returns:
            ComputeConfigListResult with 'results' (list), 'next_token' (str|None), and 'count' (int)

        Raises:
            ValueError: If both cloud_id and cloud_name are provided
        """
        if cloud_id and cloud_name:
            raise ValueError(
                "Only one of cloud_id or cloud_name can be provided, not both."
            )

        return self._private_sdk.list_compute_configs(
            name=name,
            id=_id,
            cloud_id=cloud_id,
            cloud_name=cloud_name,
            sort_by=sort_by,
            sort_order=sort_order,
            max_items=max_items,
            next_token=next_token,
            include_shared=include_shared,
        )



from typing import Optional

from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_command
from anyscale.image._private.image_sdk import PrivateImageSDK
from anyscale.image.models import ImageBuild


_IMAGE_SDK_SINGLETON_KEY = "image_sdk"

_BUILD_EXAMPLE = """
import anyscale

containerfile = '''
FROM anyscale/ray:2.21.0-py39
RUN pip install --no-cache-dir pandas
'''

image_uri: str = anyscale.image.build(containerfile, name="mycoolimage")
"""

_BUILD_ARG_DOCSTRINGS = {
    "name": "The name of the image.",
    "containerfile": "The content of the Containerfile.",
    "ray_version": "The version of Ray to use in the image",
}


@sdk_command(
    _IMAGE_SDK_SINGLETON_KEY,
    PrivateImageSDK,
    doc_py_example=_BUILD_EXAMPLE,
    arg_docstrings=_BUILD_ARG_DOCSTRINGS,
)
def build(
    containerfile: str,
    *,
    name: str,
    ray_version: Optional[str] = None,
    _private_sdk: Optional[PrivateImageSDK] = None,
) -> str:
    """Build an image from a Containerfile.

    Returns the URI of the image.
    """
    return _private_sdk.build_image_from_containerfile_with_image_uri(  # type: ignore
        name, containerfile, ray_version=ray_version
    )


_GET_EXAMPLE = """
import anyscale

image_status = anyscale.image.get(name="mycoolimage")
"""

_GET_ARG_DOCSTRINGS = {
    "name": (
        "Get the details of an image.\n\n"
        "The name can contain an optional version, e.g., 'name:version'. "
        "If no version is provided, the latest one will be used.\n\n"
    )
}


@sdk_command(
    _IMAGE_SDK_SINGLETON_KEY,
    PrivateImageSDK,
    doc_py_example=_GET_EXAMPLE,
    arg_docstrings=_GET_ARG_DOCSTRINGS,
)
def get(*, name: str, _private_sdk: Optional[PrivateImageSDK] = None) -> ImageBuild:
    """The name can contain an optional version tag, i.e., 'name:version'.

    If no version is provided, the latest one will be returned.
    """
    return _private_sdk.get(name)  # type: ignore


_LIST_EXAMPLE = """
import anyscale

for image in anyscale.image.list(max_items=5):
    print(image.name, image.latest_build_revision)
"""

_LIST_ARG_DOCSTRINGS = {
    "image_id": "If provided, returns just the image with this ID wrapped in a one-page iterator.",
    "name": "Substring to match against the image name.",
    "image_name": "Substring to match against the resolved image URI (BYOD images only).",
    "project": "Name of the project that owns the image.",
    "creator_id": "Filter images by the creator user ID.",
    "include_archived": "Include archived images (default: False).",
    "include_anonymous": "Include anonymous (workspace-scoped) images (default: False).",
    "max_items": "Maximum total number of items to yield (default: iterate all).",
    "page_size": "Number of items to fetch per API request (default: API default).",
}


@sdk_command(
    _IMAGE_SDK_SINGLETON_KEY,
    PrivateImageSDK,
    doc_py_example=_LIST_EXAMPLE,
    arg_docstrings=_LIST_ARG_DOCSTRINGS,
)
def list(  # noqa: A001
    *,
    image_id: Optional[str] = None,
    name: Optional[str] = None,
    image_name: Optional[str] = None,
    project: Optional[str] = None,
    creator_id: Optional[str] = None,
    include_archived: bool = False,
    include_anonymous: bool = False,
    max_items: Optional[int] = None,
    page_size: Optional[int] = None,
    _private_sdk: Optional[PrivateImageSDK] = None,
) -> ResultIterator[ImageBuild]:
    """List images or fetch a single image by ID."""
    return _private_sdk.list(  # type: ignore
        image_id=image_id,
        name=name,
        image_name=image_name,
        project=project,
        creator_id=creator_id,
        include_archived=include_archived,
        include_anonymous=include_anonymous,
        max_items=max_items,
        page_size=page_size,
    )


_REGISTER_EXAMPLE = """
import anyscale

image_uri: str = anyscale.image.register("docker.io/myuser/myimage:v2", name="mycoolimage")
"""

_REGISTER_ARG_DOCSTRINGS = {
    "image_uri": "The URI of the BYOD image to register.",
    "name": "Name for the container image. If the name already exists, a new version will be built. Otherwise, a new container image will be created.",
    "ray_version": "The Ray version (X.Y.Z) specified for this image specified by either an image URI or a containerfile. If you don't specify a Ray version, Anyscale defaults to the latest Ray version available at the time of the Anyscale CLI/SDK release.",
    "registry_login_secret": "Name or identifier of the secret containing credentials to authenticate to the docker registry hosting the image.",  # pragma: allowlist secret
}


@sdk_command(
    _IMAGE_SDK_SINGLETON_KEY,
    PrivateImageSDK,
    doc_py_example=_REGISTER_EXAMPLE,
    arg_docstrings=_REGISTER_ARG_DOCSTRINGS,
)
def register(
    image_uri: str,
    *,
    name: str,
    ray_version: Optional[str] = None,
    registry_login_secret: Optional[str] = None,
    _private_sdk: Optional[PrivateImageSDK] = None,
) -> str:
    """
    Registers a BYOD image with a container image name.

    Returns the URI of the image.
    """
    return _private_sdk.register_byod_image_with_name(  # type: ignore
        image_uri,
        registry_login_secret=registry_login_secret,
        ray_version=ray_version,
        name=name,
    )


_ARCHIVE_EXAMPLE = """
import anyscale

anyscale.image.archive(name="my-old-image")
"""

_ARCHIVE_ARG_DOCSTRINGS = {
    "name": "Name of the image to archive. Can include an optional version tag (e.g., 'name:version').",
}


@sdk_command(
    _IMAGE_SDK_SINGLETON_KEY,
    PrivateImageSDK,
    doc_py_example=_ARCHIVE_EXAMPLE,
    arg_docstrings=_ARCHIVE_ARG_DOCSTRINGS,
)
def archive(name: str, *, _private_sdk: Optional[PrivateImageSDK] = None,) -> None:
    """Archive an image and all of its versions.

    Once archived, the image name will no longer be usable in the organization.
    Archived images can still be viewed using `include_archived=True` in list().
    """
    return _private_sdk.archive(name=name)  # type: ignore



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class CreateWorkspaceTemplateVersion(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'image_uri': 'str',
        'compute_configs': 'dict(str, str)',
        'artifacts': 'WorkspaceSystemArtifacts',
        'source_workspace_id': 'str',
        'idle_termination_minutes': 'int'
    }

    attribute_map = {
        'image_uri': 'image_uri',
        'compute_configs': 'compute_configs',
        'artifacts': 'artifacts',
        'source_workspace_id': 'source_workspace_id',
        'idle_termination_minutes': 'idle_termination_minutes'
    }

    def __init__(self, image_uri=None, compute_configs=None, artifacts=None, source_workspace_id=None, idle_termination_minutes=None, local_vars_configuration=None):  # noqa: E501
        """CreateWorkspaceTemplateVersion - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._image_uri = None
        self._compute_configs = None
        self._artifacts = None
        self._source_workspace_id = None
        self._idle_termination_minutes = None
        self.discriminator = None

        if image_uri is not None:
            self.image_uri = image_uri
        if compute_configs is not None:
            self.compute_configs = compute_configs
        if artifacts is not None:
            self.artifacts = artifacts
        if source_workspace_id is not None:
            self.source_workspace_id = source_workspace_id
        if idle_termination_minutes is not None:
            self.idle_termination_minutes = idle_termination_minutes

    @property
    def image_uri(self):
        """Gets the image_uri of this CreateWorkspaceTemplateVersion.  # noqa: E501

        Docker image URI for this version  # noqa: E501

        :return: The image_uri of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :rtype: str
        """
        return self._image_uri

    @image_uri.setter
    def image_uri(self, image_uri):
        """Sets the image_uri of this CreateWorkspaceTemplateVersion.

        Docker image URI for this version  # noqa: E501

        :param image_uri: The image_uri of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :type: str
        """

        self._image_uri = image_uri

    @property
    def compute_configs(self):
        """Gets the compute_configs of this CreateWorkspaceTemplateVersion.  # noqa: E501

        Cloud-specific compute config mapping  # noqa: E501

        :return: The compute_configs of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._compute_configs

    @compute_configs.setter
    def compute_configs(self, compute_configs):
        """Sets the compute_configs of this CreateWorkspaceTemplateVersion.

        Cloud-specific compute config mapping  # noqa: E501

        :param compute_configs: The compute_configs of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :type: dict(str, str)
        """

        self._compute_configs = compute_configs

    @property
    def artifacts(self):
        """Gets the artifacts of this CreateWorkspaceTemplateVersion.  # noqa: E501

        Artifacts configuration for this version  # noqa: E501

        :return: The artifacts of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :rtype: WorkspaceSystemArtifacts
        """
        return self._artifacts

    @artifacts.setter
    def artifacts(self, artifacts):
        """Sets the artifacts of this CreateWorkspaceTemplateVersion.

        Artifacts configuration for this version  # noqa: E501

        :param artifacts: The artifacts of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :type: WorkspaceSystemArtifacts
        """

        self._artifacts = artifacts

    @property
    def source_workspace_id(self):
        """Gets the source_workspace_id of this CreateWorkspaceTemplateVersion.  # noqa: E501

        Source workspace ID to copy image_uri, compute_configs, and artifacts from. If both source_workspace_id and artifacts are provided, artifacts will take precedence.  # noqa: E501

        :return: The source_workspace_id of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :rtype: str
        """
        return self._source_workspace_id

    @source_workspace_id.setter
    def source_workspace_id(self, source_workspace_id):
        """Sets the source_workspace_id of this CreateWorkspaceTemplateVersion.

        Source workspace ID to copy image_uri, compute_configs, and artifacts from. If both source_workspace_id and artifacts are provided, artifacts will take precedence.  # noqa: E501

        :param source_workspace_id: The source_workspace_id of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :type: str
        """

        self._source_workspace_id = source_workspace_id

    @property
    def idle_termination_minutes(self):
        """Gets the idle_termination_minutes of this CreateWorkspaceTemplateVersion.  # noqa: E501

        Idle termination minutes for this version  # noqa: E501

        :return: The idle_termination_minutes of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :rtype: int
        """
        return self._idle_termination_minutes

    @idle_termination_minutes.setter
    def idle_termination_minutes(self, idle_termination_minutes):
        """Sets the idle_termination_minutes of this CreateWorkspaceTemplateVersion.

        Idle termination minutes for this version  # noqa: E501

        :param idle_termination_minutes: The idle_termination_minutes of this CreateWorkspaceTemplateVersion.  # noqa: E501
        :type: int
        """

        self._idle_termination_minutes = idle_termination_minutes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateWorkspaceTemplateVersion):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CreateWorkspaceTemplateVersion):
            return True

        return self.to_dict() != other.to_dict()



import difflib
import json
import pprint
from typing import Any, Dict, List, Optional

import boto3
from botocore.exceptions import ClientError
from click import ClickException

from anyscale.aws_iam_policies import (
    AMAZON_ECR_READONLY_ACCESS_POLICY_NAME,
    ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE,
    ANYSCALE_IAM_PERMISSIONS_SERVICE_STEADY_STATE,
    get_anyscale_aws_iam_assume_role_policy,
    get_anyscale_iam_permissions_ec2_restricted,
)
from anyscale.cli_logger import CloudSetupLogger
from anyscale.client.openapi_client.models.aws_memory_db_cluster_config import (
    AWSMemoryDBClusterConfig,
)
from anyscale.client.openapi_client.models.cloud_analytics_event_cloud_resource import (
    CloudAnalyticsEventCloudResource,
)
from anyscale.client.openapi_client.models.subnet_id_with_availability_zone_aws import (
    SubnetIdWithAvailabilityZoneAWS,
)
from anyscale.shared_anyscale_utils.aws import AwsRoleArn
from anyscale.shared_anyscale_utils.conf import ANYSCALE_CORS_ORIGIN
from anyscale.util import (  # pylint:disable=private-import
    _get_subnet,
    contains_control_plane_role,
    filter_actions_associated_with_role,
    filter_actions_from_policy_document,
    verify_data_plane_role_assume_role_policy,
)
from anyscale.utils.cloud_utils import CloudSetupError
from anyscale.utils.network_verification import (
    AWS_SUBNET_CAPACITY,
    AWS_VPC_CAPACITY,
)
from anyscale.utils.s3 import verify_s3_access


# This needs to be kept in sync with the Ray autoscaler in
# https://github.com/ray-project/ray/blob/eb9c5d8fa70b1c360b821f82c7697e39ef94b25e/python/ray/autoscaler/_private/aws/config.py
# It should go away with the SSM refactor.
DEFAULT_RAY_IAM_ROLE = "ray-autoscaler-v1"

S3_ARN_PREFIX = "arn:aws:s3:::"
S3_STORAGE_PREFIX = "s3://"
GCS_STORAGE_PREFIX = "gs://"

HTTPS_INGRESS_PORT = 443


def compare_dicts_diff(d1: Dict[Any, Any], d2: Dict[Any, Any]) -> str:
    """Returns a string representation of the difference of the two dictionaries.
    Example:

    Input:
    print(compare_dicts_diff({"a": {"c": 1}, "b": 2}, {"a": {"c": 2}, "d": 3}))

    Output:
    - {'a': {'c': 1}, 'b': 2}
    ?             ^    ^   ^

    + {'a': {'c': 2}, 'd': 3}
    ?             ^    ^   ^
    """

    return "\n" + "\n".join(
        difflib.ndiff(pprint.pformat(d1).splitlines(), pprint.pformat(d2).splitlines())
    )


AWS_RESOURCE_DICT: Dict[str, str] = {
    "VPC": CloudAnalyticsEventCloudResource.AWS_VPC,
    "Subnet": CloudAnalyticsEventCloudResource.AWS_SUBNET,
    "Security group": CloudAnalyticsEventCloudResource.AWS_SECURITY_GROUP,
    "S3 bucket": CloudAnalyticsEventCloudResource.AWS_S3_BUCKET,
    "EFS": CloudAnalyticsEventCloudResource.AWS_EFS,
    "CloudFormation stack": CloudAnalyticsEventCloudResource.AWS_CLOUDFORMATION,
    "MemoryDB cluster": CloudAnalyticsEventCloudResource.AWS_MEMORYDB,
}


def log_resource_not_found_error(
    resource_name: str, resource_id: str, logger: CloudSetupLogger
) -> None:
    resource = AWS_RESOURCE_DICT.get(resource_name)
    if resource:
        logger.log_resource_error(resource, CloudSetupError.RESOURCE_NOT_FOUND)
    logger.error(
        f"Could not find {resource_name} with id {resource_id}. Please validate that you're using the correct AWS account/credentials and that the resource values are correct"
    )


def verify_aws_vpc(
    aws_vpc_id: Optional[str],
    boto3_session: boto3.Session,
    logger: CloudSetupLogger,
    ignore_capacity_errors: bool = False,  # TODO: Probably don't do this forever. Its kinda hacky
    strict: bool = False,  # strict is currently unused # noqa: ARG001
) -> bool:
    logger.info("Verifying VPC ...")
    if not aws_vpc_id:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_VPC,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing VPC id.")
        return False

    ec2 = boto3_session.resource("ec2")
    vpc = ec2.Vpc(aws_vpc_id)

    # Verify the VPC exists
    try:
        vpc.load()
    except ClientError as e:
        if e.response["Error"]["Code"] == "InvalidVpcID.NotFound":
            log_resource_not_found_error("VPC", aws_vpc_id, logger)
            return False
        else:
            logger.log_resource_exception(CloudAnalyticsEventCloudResource.AWS_VPC, e)
        raise e

    # Verify that the VPC has "enough" capacity.
    if ignore_capacity_errors or AWS_VPC_CAPACITY.verify_network_capacity(
        cidr_block_str=vpc.cidr_block, resource_name=vpc.id, logger=logger
    ):
        logger.info(f"VPC {vpc.id} verification succeeded.")
        return True
    return False


def _get_subnets_from_subnet_ids(
    subnet_ids: List[str], region: str, logger: CloudSetupLogger
) -> List[Any]:
    return [
        _get_subnet(subnet_arn=subnet_id, region=region, logger=logger)
        for subnet_id in subnet_ids
    ]


def verify_aws_subnets(  # noqa: PLR0911, PLR0912
    aws_vpc_id: Optional[str],
    aws_subnet_ids: List[str],
    region: str,
    is_private_network: bool,
    logger: CloudSetupLogger,
    ignore_capacity_errors: bool = False,  # TODO: Probably don't do this forever. Its kinda hacky
    strict: bool = False,
) -> bool:
    """Verify the subnets cloud resource of a cloud."""

    logger.info("Verifying subnets ...")

    if not aws_vpc_id:
        logger.error("Missing VPC ID.")
        return False

    if not aws_subnet_ids:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_SUBNET,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing subnet IDs.")
        return False

    # We must have at least 2 subnets since services requires 2 different subnets to setup ALB.
    if len(aws_subnet_ids) < 2:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_SUBNET, CloudSetupError.ONLY_ONE_SUBNET
        )
        logger.error(
            "Need at least 2 subnets for a cloud. This is required for Anyscale services to function properly."
        )
        return False

    subnets = _get_subnets_from_subnet_ids(
        subnet_ids=aws_subnet_ids, region=region, logger=logger
    )
    subnet_azs = set()

    for subnet, subnet_id in zip(subnets, aws_subnet_ids):
        # Verify subnet exists
        if not subnet:
            log_resource_not_found_error("Subnet", subnet_id, logger)
            return False

        # Verify the Subnet has "enough" capacity.
        if (
            not AWS_SUBNET_CAPACITY.verify_network_capacity(
                cidr_block_str=subnet.cidr_block, resource_name=subnet.id, logger=logger
            )
            and not ignore_capacity_errors
        ):
            return False

        # Verify that the subnet is in the provided VPC all of these are in the same VPC.
        if subnet.vpc_id != aws_vpc_id:
            logger.log_resource_error(
                CloudAnalyticsEventCloudResource.AWS_SUBNET,
                CloudSetupError.SUBNET_NOT_IN_VPC,
            )
            logger.error(
                f"The subnet {subnet_id} is not in a vpc of this cloud. The vpc of this subnet is {subnet.vpc_id} and the vpc of this cloud is {aws_vpc_id}."
            )
            return False

        # Verify that the subnet is auto-assigning public IP addresses if it's not private.
        if not is_private_network and not subnet.map_public_ip_on_launch:
            logger.warning(
                f"The subnet {subnet_id} does not have the 'Auto-assign Public IP' option enabled. This is not currently supported."
            )
            if strict:
                return False
        if is_private_network and subnet.map_public_ip_on_launch:
            logger.warning(
                f"The private subnet {subnet_id} shouldn't have 'Auto-assign Public IP' option enabled."
                "Please remove the `--private-network` flag if you don't want to deploy your Ray clusters in private subnets."
            )
            if strict:
                return False

        # Success!
        logger.info(f"Subnet {subnet.id}'s verification succeeded.")
        subnet_azs.add(subnet.availability_zone)

    if len(subnet_azs) < 2:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_SUBNET, CloudSetupError.ONLY_ONE_AZ,
        )
        logger.error(
            "Subnets should be in at least 2 Availability Zones. This is required for Anyscale services to function properly."
        )
        return False

    logger.info(f"Subnets {aws_subnet_ids} verification succeeded.")
    return True


def associate_aws_subnets_with_azs(
    aws_subnet_ids: List[str], region: str, logger: CloudSetupLogger
) -> List[SubnetIdWithAvailabilityZoneAWS]:
    """This function combines the subnets with its availability zone.
    """

    subnets = _get_subnets_from_subnet_ids(
        subnet_ids=aws_subnet_ids, region=region, logger=logger
    )

    # combine subnet and its availability zone
    subnet_ids_with_availability_zones = [
        SubnetIdWithAvailabilityZoneAWS(
            subnet_id=subnet.id, availability_zone=subnet.availability_zone,
        )
        for subnet in subnets
    ]

    return subnet_ids_with_availability_zones


def _get_role_from_arn(
    role_arn: str, boto3_session: boto3.Session, logger: CloudSetupLogger,
) -> Any:
    iam = boto3_session.resource("iam")

    # Validate the role exists.
    # `.load()` will throw an exception if the Role does not exist.
    try:
        role = iam.Role(AwsRoleArn.from_string(role_arn).to_role_name())
        role.load()
        return role
    except ClientError as e:
        logger.log_resource_exception(CloudAnalyticsEventCloudResource.AWS_IAM_ROLE, e)
        if e.response["Error"]["Code"] == "NoSuchEntity":
            logger.error(f"Could not find role: {role.name if role else 'unknown'}")
            return None
        raise e


def verify_aws_iam_roles(  # noqa: PLR0911, PLR0912
    control_plane_role: Optional[str],
    data_plane_role: Optional[str],
    boto3_session: boto3.Session,
    anyscale_aws_account: str,
    logger: CloudSetupLogger,
    cloud_id: str,
    strict: bool = False,
    _use_strict_iam_permissions: bool = False,
) -> bool:

    logger.info("Verifying IAM roles ...")
    if not control_plane_role:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_IAM_ROLE,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing Anyscale IAM role.")
        return False
    if not data_plane_role:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_IAM_ROLE,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing Cluster Node IAM role.")
        return False
    accounts = {
        AwsRoleArn.from_string(role).account_id
        for role in [control_plane_role, data_plane_role]
    }
    if len(accounts) != 1:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_IAM_ROLE,
            CloudSetupError.IAM_ROLE_ACCOUNT_MISMATCH,
        )
        logger.error(
            f"All IAM roles must be in the same AWS account: {control_plane_role}, {data_plane_role}"
        )
        return False

    # verifying control plane role: anyscale iam role
    anyscale_iam_role = _get_role_from_arn(control_plane_role, boto3_session, logger)
    if not anyscale_iam_role:
        return False
    assume_role_policy_document = anyscale_iam_role.assume_role_policy_document
    if not contains_control_plane_role(
        assume_role_policy_document=assume_role_policy_document,
        anyscale_aws_account=anyscale_aws_account,
    ):
        logger.warning(
            f"Anyscale IAM role {anyscale_iam_role.arn} does not contain expected assume role policy. It must allow assume role from arn:aws:iam::{anyscale_aws_account}:root."
        )
        expected_assume_role_policy_document = get_anyscale_aws_iam_assume_role_policy(
            anyscale_aws_account=anyscale_aws_account
        )
        logger.warning(
            compare_dicts_diff(
                assume_role_policy_document, expected_assume_role_policy_document
            )
        )
        if strict:
            return False

    # Verify EC2 steady state permissions
    # If permissions are missing, log warning message
    anyscale_iam_permissions_ec2 = ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE
    if _use_strict_iam_permissions:
        anyscale_iam_permissions_ec2 = get_anyscale_iam_permissions_ec2_restricted(
            cloud_id
        )

    allow_actions_expected = filter_actions_from_policy_document(
        anyscale_iam_permissions_ec2
    )
    allow_actions_on_role = filter_actions_associated_with_role(
        boto3_session, anyscale_iam_role
    )
    allow_actions_missing = allow_actions_expected - allow_actions_on_role

    if allow_actions_missing:
        logger.warning(
            f"IAM role {anyscale_iam_role.arn} does not have sufficient permissions for cluster management. We suggest adding these actions to ensure that cluster management works properly: {allow_actions_missing}. "
        )

    # Verify Service Steady State permissions
    # If service permissions are missing, display confirmation message to user if they would like to continue
    allow_actions_expected = filter_actions_from_policy_document(
        ANYSCALE_IAM_PERMISSIONS_SERVICE_STEADY_STATE
    )
    allow_actions_missing = allow_actions_expected - allow_actions_on_role
    if allow_actions_missing:
        logger.print_red_error_message(
            "[SERVICES V2] Permissions are missing to enable services v2 "
        )
        logger.confirm_missing_permission(
            f"For IAM role {anyscale_iam_role.arn}, we suggest adding the following actions:\n{pprint.pformat(allow_actions_missing)}.\n"
        )
        if strict:
            return False

    # verifying data plane role: ray autoscaler role
    cluster_node_role = _get_role_from_arn(data_plane_role, boto3_session, logger)
    if not cluster_node_role:
        return False
    assume_role_policy_document = cluster_node_role.assume_role_policy_document
    if not verify_data_plane_role_assume_role_policy(
        assume_role_policy_document=assume_role_policy_document,
    ):
        logger.error(
            f"Cluster Node IAM role {cluster_node_role.arn} does not contain expected assume role policy. It must give trust to AWS service EC2."
        )
        return False

    policy_names = [
        policy.policy_name for policy in cluster_node_role.attached_policies.all()
    ]
    if AMAZON_ECR_READONLY_ACCESS_POLICY_NAME not in policy_names:
        logger.warning(
            f"Dataplane role {cluster_node_role.arn} does not contain policy {AMAZON_ECR_READONLY_ACCESS_POLICY_NAME}. This is safe to ignore if you are not pulling custom Docker Images from an ECR repository."
        )
        if strict:
            return False

    if (
        len(
            [
                profile
                for profile in cluster_node_role.instance_profiles.all()
                if profile.name == cluster_node_role.name
            ]
        )
        == 0
    ):
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_IAM_ROLE,
            CloudSetupError.INSTANCE_PROFILE_NOT_FOUND,
        )
        logger.error(
            f"Dataplane role {cluster_node_role.arn} is required to have an instance profile with the name {cluster_node_role.name}."
            "\nPlease create this instance profile and associate it to the role."
        )
        return False

    logger.info(
        f"IAM roles {control_plane_role}, {data_plane_role} verification succeeded."
    )
    return True


def is_internal_communication_allowed(
    ip_permissions: List, aws_security_group_ids: List[str]
) -> bool:
    """
    This is a helper function to check if the security group allows internal communication.
    It can be used for both inbound and outbound permissions.
    """
    sg_rule_with_self = []  # type: ignore
    for sg_rule in ip_permissions:
        if sg_rule.get("IpProtocol") == "-1":
            sg_rule_with_self.extend(sg_rule.get("UserIdGroupPairs"))  # type: ignore
    return any(
        sg_rule.get("GroupId") in aws_security_group_ids
        for sg_rule in sg_rule_with_self
    )


def verify_aws_security_groups(  # noqa: PLR0912, PLR0911
    aws_security_group_ids: Optional[List[str]],
    boto3_session: boto3.Session,
    logger: CloudSetupLogger,
    strict: bool = False,
) -> bool:
    logger.info("Verifying security groups ...")
    if not aws_security_group_ids:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_SECURITY_GROUP,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing security group IDs.")
        return False

    ec2 = boto3_session.resource("ec2")

    anyscale_security_groups = []
    for anyscale_security_group_id in aws_security_group_ids:
        anyscale_security_group = ec2.SecurityGroup(anyscale_security_group_id)
        try:
            anyscale_security_group.load()
        except ClientError as e:
            if e.response["Error"]["Code"] == "InvalidGroup.NotFound":
                log_resource_not_found_error(
                    "Security group", anyscale_security_group_id, logger
                )
                return False
            else:
                logger.log_resource_exception(
                    CloudAnalyticsEventCloudResource.AWS_SECURITY_GROUP, e
                )
            raise e
        anyscale_security_groups.append(anyscale_security_group)

    inbound_ip_permissions = [
        ip_permission
        for anyscale_security_group in anyscale_security_groups
        for ip_permission in anyscale_security_group.ip_permissions
    ]
    outbound_ip_permissions = [
        ip_permission
        for anyscale_security_group in anyscale_security_groups
        for ip_permission in anyscale_security_group.ip_permissions_egress
    ]
    inbound_ip_permissions_with_specific_port = {
        ip_permission["FromPort"]
        for ip_permission in inbound_ip_permissions
        if "FromPort" in ip_permission
    }

    # Check inbound permissions
    if not any(
        inbound_ip_permission_port == HTTPS_INGRESS_PORT
        for inbound_ip_permission_port in inbound_ip_permissions_with_specific_port
    ):
        logger.warning(
            f"Security groups {aws_security_group_ids} do not contain inbound permission for port {HTTPS_INGRESS_PORT}. This port is used for interaction with the clusters from Anyscale UI."
        )
        if strict:
            return False

    expected_open_ports = [
        HTTPS_INGRESS_PORT,
        22,
    ]  # 22 was previously used for SSH but is no longer required.
    if len(inbound_ip_permissions_with_specific_port) > len(expected_open_ports):
        logger.warning(
            f"Security groups {aws_security_group_ids} allows access to more than {expected_open_ports}. This may not be safe by default."
        )
        if strict:
            return False

    # Check internal communication is allowed
    if not is_internal_communication_allowed(
        inbound_ip_permissions, aws_security_group_ids
    ):
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_SECURITY_GROUP,
            CloudSetupError.INTERNAL_COMMUNICATION_NOT_ALLOWED,
        )
        logger.error(
            f"Security groups {aws_security_group_ids} do not contain inbound permission for all ports for traffic from the same security group."
        )
        return False

    # Check outbound permissions
    if not is_internal_communication_allowed(
        outbound_ip_permissions, aws_security_group_ids
    ):
        logger.warning(
            f"Security groups {aws_security_group_ids} do not contain outbound permission for all protocols for traffic from the same security group. "
            f"This is required for certain network device such as EFA."
        )
        if strict:
            return False

    logger.info(f"Security group {aws_security_group_ids} verification succeeded.")
    return True


def verify_aws_s3(  # noqa: PLR0911, PLR0912
    aws_s3_id: Optional[str],
    control_plane_role: Optional[str],
    data_plane_role: Optional[str],
    boto3_session: boto3.Session,
    region: str,
    logger: CloudSetupLogger,
    strict: bool = False,
) -> bool:
    logger.info("Verifying S3 ...")
    if not aws_s3_id:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_S3_BUCKET,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing S3 ID.")
        return False

    s3 = boto3_session.resource("s3")
    bucket_name = aws_s3_id.split(":")[-1]
    s3_bucket = s3.Bucket(bucket_name)

    # Check for the existence of `creation_date` because this incurs a `list_bucket` call.
    # Calling `.load()` WILL NOT ERROR in cases where the caller does not have access to the bucket.
    if s3_bucket.creation_date is None:
        log_resource_not_found_error("S3 bucket", aws_s3_id, logger)
        return False

    has_correct_cors_rule = False
    """
    Verify CORS rules. The correct CORS rule should look like:
    [{
        "AllowedHeaders": [
            "*"
        ],
        "AllowedMethods": [
            "GET"
        ],
        "AllowedOrigins": [
            "https://console.anyscale-staging.com"
        ],
        "ExposeHeaders": []
    }]
    """

    try:
        cors_rules = s3_bucket.Cors().cors_rules
    except ClientError as e:
        logger.log_resource_exception(CloudAnalyticsEventCloudResource.AWS_S3_BUCKET, e)
        if e.response["Error"]["Code"] == "NoSuchCORSConfiguration":
            logger.warning(
                f"S3 bucket {bucket_name} does not have CORS rules. This is safe to ignore if you are not using Anyscale UI. Otherwise please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/cloud-deployment/aws/manage-clouds#s3"
            )
            cors_rules = []
            if strict:
                return False
        else:
            raise e

    for rule in cors_rules:
        if not isinstance(rule, dict):
            logger.log_resource_error(
                CloudAnalyticsEventCloudResource.AWS_S3_BUCKET,
                CloudSetupError.MALFORMED_CORS_RULE,
            )
            logger.error(f"Malformed CORS rule {rule} for your S3 bucket.")
            return False
        has_correct_cors_rule = (
            ANYSCALE_CORS_ORIGIN in rule.get("AllowedOrigins", [])
            and "*" in rule.get("AllowedHeaders", [])
            and "GET" in rule.get("AllowedMethods", [])
        )

    if not has_correct_cors_rule:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_S3_BUCKET,
            CloudSetupError.INCORRECT_CORS_RULE,
        )
        logger.warning(
            f"S3 bucket {bucket_name} does not have the correct CORS rule for Anyscale. This is safe to ignore if you are not using Anyscale UI. Otherwise please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/cloud-deployment/aws/manage-clouds#appendix-detailed-resource-requirements"
        )
        if strict:
            return False

    returned_bucket_location = boto3_session.client("s3").get_bucket_location(
        Bucket=bucket_name
    )["LocationConstraint"]

    # LocationConstraint is `None` if the bucket is located in us-east-1
    bucket_region = returned_bucket_location or "us-east-1"
    if bucket_region != region:
        logger.warning(
            f"S3 bucket {bucket_name} is in region {bucket_region}, but this cloud is being set up in {region}."
            "This can result in degraded cluster launch & logging performance as well as additional cross-region costs."
        )
        if strict:
            return False

    if not control_plane_role or not data_plane_role:
        return False

    role = _get_role_from_arn(control_plane_role, boto3_session, logger)
    if not verify_s3_access(boto3_session, s3_bucket, role, logger):
        logger.warning(
            f"S3 Bucket {bucket_name} does not appear to have correct permissions for the Anyscale Control Plane role {role.name}"
        )
        if strict:
            return False

    role = _get_role_from_arn(data_plane_role, boto3_session, logger)
    if not verify_s3_access(boto3_session, s3_bucket, role, logger):
        logger.warning(
            f"S3 Bucket {bucket_name} does not appear to have correct permissions for the Data Plane role {role.name}"
        )
        if strict:
            return False
    logger.info(f"S3 {aws_s3_id} verification succeeded.")
    return True


def _get_network_interfaces_from_mount_targets(
    mount_targets_response: dict, boto3_session: Any, logger: CloudSetupLogger
) -> List[Any]:
    ec2 = boto3_session.resource("ec2")
    network_interfaces = []
    for network_interface_id in [
        mount_target["NetworkInterfaceId"]
        for mount_target in mount_targets_response["MountTargets"]
    ]:
        network_interface = ec2.NetworkInterface(network_interface_id)
        try:
            network_interface.load()
        except ClientError as e:
            logger.warning(f"Network interface loading error: {e}")
            continue
        network_interfaces.append(network_interface)
    return network_interfaces


def verify_aws_efs(  # noqa: PLR0911, PLR0912, C901
    aws_efs_id: Optional[str],
    aws_efs_mount_target_ips: Optional[List[str]],
    aws_subnet_ids: List[str],
    aws_security_groups: Optional[List[str]],
    boto3_session: boto3.Session,
    logger: CloudSetupLogger,
    strict: bool = False,
) -> bool:
    logger.info("Verifying EFS ...")
    if not aws_efs_id:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_EFS,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing EFS ID.")
        return False
    if not aws_subnet_ids:
        logger.error("Missing subnet IDs.")
        return False
    if not aws_security_groups:
        logger.error("Missing security group IDs.")
        return False

    client = boto3_session.client("efs")
    try:
        file_systems_response = client.describe_file_systems(FileSystemId=aws_efs_id)
    except ClientError as e:
        if e.response["Error"]["Code"] == "FileSystemNotFound":
            log_resource_not_found_error("EFS", aws_efs_id, logger)
            return False
        else:
            logger.log_resource_exception(CloudAnalyticsEventCloudResource.AWS_EFS, e)
        raise e

    if len(file_systems_response.get("FileSystems", [])) == 0:
        log_resource_not_found_error("EFS", aws_efs_id, logger)
        return False

    # verify that there is a mount target for each subnet and security group
    mount_targets_response = client.describe_mount_targets(FileSystemId=aws_efs_id)
    mount_targets = mount_targets_response.get("MountTargets")
    if not mount_targets:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_EFS,
            CloudSetupError.MOUNT_TARGET_NOT_FOUND,
        )
        logger.error(f"EFS with id {aws_efs_id} does not contain mount targets.")
        return False

    # verify the mount target ID stored in our database is still valid
    mount_target_ips = [mount_target["IpAddress"] for mount_target in mount_targets]
    if aws_efs_mount_target_ips and any(
        ip not in mount_target_ips for ip in aws_efs_mount_target_ips
    ):
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_EFS,
            CloudSetupError.INVALID_MOUNT_TARGET,
        )
        logger.error(
            f"Mount target registered with the cloud no longer exists. EFS ID: {aws_efs_id} IP address: {aws_efs_mount_target_ips}. Please make sure you have the correct AWS credentials set. If the EFS mount target has been deleted, please recreate the cloud or contact Anyscale for support."
        )
        logger.error(
            f"Valid mount target IPs for EFS ID {aws_efs_id} are {mount_target_ips}. "
            "If this happens during cloud edit, ensure that: "
            "1) If only editing aws_efs_mount_target_ip, it belongs to the existing EFS ID. "
            "2) If editing both efs_id and efs_mount_target_ip, the new IP is a valid target for the new efs_id."
        )
        return False

    network_interfaces = _get_network_interfaces_from_mount_targets(
        mount_targets_response, boto3_session, logger
    )

    expected_security_group_id = aws_security_groups[0]

    # Condition 1: No matching network interface in EFS mount targets for a subnet.
    # - 1.1: EFS has mount targets in other subnets.
    #        Subnet communicates with EFS through cross AZ, incurring cross AZ costs. (warning)
    # - 1.2: EFS doesn't have mount target, the subnet cannot communicate with EFS. (error)
    #   (note) A previous check ensures EFS has mount targets if this point is reached.
    # --------------------------------------------------------------------------------------------------------------
    # Condition 2: Subnet has a matching network interface in EFS mount targets.
    # - 2.1: Network interface has a registered security group. (happy path)
    # - 2.2: Network interface lacks a registered security group but has another security group.
    #   If configured correctly, subnet can still communicate with EFS. (warning)
    # --------------------------------------------------------------------------------------------------------------
    for subnet_id in aws_subnet_ids:
        contains_subnet_id = False
        contains_registered_security_group = False
        for network_interface in network_interfaces:
            network_interface_security_group_ids = [
                group["GroupId"]
                for group in network_interface.groups
                if group.get("GroupId")
            ]
            if network_interface.subnet_id == subnet_id:
                contains_subnet_id = True
                if expected_security_group_id in network_interface_security_group_ids:
                    contains_registered_security_group = True
                    break
        if not contains_subnet_id:
            # condition 1.1.
            logger.warning(
                f"EFS with id {aws_efs_id} does not contain a mount target with the subnet {subnet_id}, which might introduce cross AZ networking cost."
            )
            if strict:
                return False
        elif not contains_registered_security_group:
            # condition 2.2.
            logger.warning(
                f"EFS with id {aws_efs_id} does not contain a mount target with the subnet {subnet_id} and security group id {aws_security_groups[0]}. This misconfiguration might pose security risks and incur connection issues, preventing the EFS from working as expected."
            )
            if strict:
                return False
    try:
        backup_policy_response = client.describe_backup_policy(FileSystemId=aws_efs_id)
        backup_policy_status = backup_policy_response.get("BackupPolicy", {}).get(
            "Status", ""
        )
        if backup_policy_status != "ENABLED":
            logger.warning(f"EFS {aws_efs_id} backup policy is not enabled.")
            if strict:
                return False
    except ClientError as e:
        if e.response["Error"]["Code"] == "PolicyNotFound":
            logger.warning(f"EFS {aws_efs_id} backup policy not found.")
            if strict:
                return False
        else:
            raise e

    # Verify efs policy
    if not _verify_aws_efs_policy(boto3_session, aws_efs_id, logger, strict):
        return False

    logger.info(f"EFS {aws_efs_id} verification succeeded.")
    return True


def _verify_aws_efs_policy(
    boto3_session: boto3.Session,
    efs_id: str,
    logger: CloudSetupLogger,
    strict: bool = False,
) -> bool:
    """
    Verify that the EFS policy has sufficient permissions.
    """
    client = boto3_session.client("efs")

    efs_policy = None
    try:
        efs_policy_response = client.describe_file_system_policy(FileSystemId=efs_id)
        efs_policy = json.loads(efs_policy_response["Policy"])
    except ClientError as e:
        if e.response["Error"]["Code"] != "PolicyNotFound":
            # If the policy is not found, we'll skip the check
            logger.error(f"Failed to describe file system policy: {e}")
            return False

    if efs_policy:
        expected_actions = [
            "elasticfilesystem:ClientRootAccess",
            "elasticfilesystem:ClientWrite",
            "elasticfilesystem:ClientMount",
        ]
        actions = filter_actions_from_policy_document(efs_policy)
        missing_actions = set(expected_actions) - set(actions)
        if missing_actions:
            logger.warning(
                f"EFS {efs_id} does not have sufficient permissions. "
                f"We suggest adding these actions to ensure that efs works properly: {missing_actions}. "
            )
            if strict:
                return False

    return True


def verify_aws_cloudformation_stack(
    aws_cloudformation_stack_id: Optional[str],
    boto3_session: boto3.Session,
    logger: CloudSetupLogger,
    strict: bool = False,  # strict is currently unused # noqa: ARG001
) -> bool:
    logger.info("Verifying CloudFormation stack ...")
    if not aws_cloudformation_stack_id:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_CLOUDFORMATION,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing CloudFormation stack id.")
        return False

    cloudformation = boto3_session.resource("cloudformation")
    stack = cloudformation.Stack(aws_cloudformation_stack_id)
    try:
        stack.load()
    except ClientError as e:
        if e.response["Error"]["Code"] == "ValidationError":
            log_resource_not_found_error(
                "CloudFormation stack", aws_cloudformation_stack_id, logger,
            )
            return False
        else:
            logger.log_resource_exception(
                CloudAnalyticsEventCloudResource.AWS_CLOUDFORMATION, e
            )
        raise e

    logger.info(
        f"CloudFormation stack {aws_cloudformation_stack_id} verification succeeded."
    )
    return True


def verify_aws_memorydb_cluster(  # noqa: PLR0911, PLR0912
    memorydb_cluster_config: Optional[AWSMemoryDBClusterConfig],
    aws_security_groups: List[str],
    aws_vpc_id: str,
    aws_subnet_ids: List[str],
    boto3_session: boto3.Session,
    logger: CloudSetupLogger,
    strict: bool = False,  # strict is currently unused # noqa: ARG001
) -> bool:
    """Verify that the MemoryDB cluster exists and is in the available state."""
    logger.info("Verifying MemoryDB ...")
    if not memorydb_cluster_config:
        logger.log_resource_error(
            CloudAnalyticsEventCloudResource.AWS_MEMORYDB,
            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
        )
        logger.error("Missing MemoryDB cluster id.")
        return False

    client = boto3_session.client("memorydb")
    try:
        response = client.describe_clusters(
            ClusterName=memorydb_cluster_config.id.split("/")[-1],
            ShowShardDetails=True,
        )
        if not response.get("Clusters"):
            log_resource_not_found_error(
                "MemoryDB cluster", memorydb_cluster_config.id, logger
            )
            return False

        # verify that the subnet group has the same security group as the cloud
        security_group_id = response["Clusters"][0].get(
            "SecurityGroups", [{"SecurityGroupId": "NOT_SPECIFIED"}]
        )[0]["SecurityGroupId"]
        if security_group_id != aws_security_groups[0]:
            logger.warning(
                f"MemoryDB cluster {memorydb_cluster_config.id} has security group {security_group_id} that is not the same as the cloud's security group {aws_security_groups[0]}."
            )
            if strict:
                return False

        # verify that the cluster is in the cloud's VPC
        subnet_group_response = client.describe_subnet_groups(
            SubnetGroupName=response["Clusters"][0]["SubnetGroupName"]
        )
        if subnet_group_response["SubnetGroups"][0]["VpcId"] != aws_vpc_id:
            logger.warning(
                f"MemoryDB cluster {memorydb_cluster_config.id} is not in the same VPC as the cloud."
            )
            if strict:
                return False

        # verify that the subnet group has the subset of subnets that the cloud has
        for subnet in subnet_group_response["SubnetGroups"][0]["Subnets"]:
            if subnet["Identifier"] not in aws_subnet_ids:
                logger.warning(
                    f"MemoryDB cluster {memorydb_cluster_config.id} has subnet {subnet['Identifier']} that is not one of the subnets in the cloud."
                )
                if strict:
                    return False

        # verify that the cluster has parameter group with the maxmemory-policy set to allkeys-lru
        parameter_response = client.describe_parameters(
            ParameterGroupName=response["Clusters"][0]["ParameterGroupName"]
        )
        for param in parameter_response["Parameters"]:
            if param["Name"] == "maxmemory-policy" and param["Value"] != "allkeys-lru":
                logger.warning(
                    f"MemoryDB cluster {memorydb_cluster_config.id} should have parameter group with maxmemory-policy set to allkeys-lru instead of {param['Value']}."
                )
                if strict:
                    return False

        # verify TLS is enabled
        if not response["Clusters"][0]["TLSEnabled"]:
            logger.error(
                f"MemoryDB cluster {memorydb_cluster_config.id} has TLS disabled. Please create a memorydb cluster with TLS enabled."
            )
            return False

        # verify that each shard in the cluster has at least 2 nodes for high availability
        for shard in response["Clusters"][0]["Shards"]:
            if len(shard["Nodes"]) < 2:
                logger.error(
                    f"MemoryDB cluster {memorydb_cluster_config.id} has shard {shard['Name']} with less than 2 nodes. This is not enough for high availability. Please make sure each shard has at least 2 nodes."
                )
                return False

    except ClientError as e:
        logger.log_resource_exception(CloudAnalyticsEventCloudResource.AWS_MEMORYDB, e)
        raise ClickException(
            f"Failed to verify MemoryDB cluster {memorydb_cluster_config.id}.\nError: {e}"
        )

    return True



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class CreateInstanceUsageBudget(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'budget_amount': 'float',
        'evaluation_period': 'InstanceUsageBudgetEvaluationPeriod',
        'budget_unit': 'InstanceUsageBudgetUnit',
        'project_id': 'str',
        'cloud_id': 'str',
        'notification_channel': 'CreateNotificationChannelRecord'
    }

    attribute_map = {
        'name': 'name',
        'budget_amount': 'budget_amount',
        'evaluation_period': 'evaluation_period',
        'budget_unit': 'budget_unit',
        'project_id': 'project_id',
        'cloud_id': 'cloud_id',
        'notification_channel': 'notification_channel'
    }

    def __init__(self, name=None, budget_amount=None, evaluation_period=None, budget_unit=None, project_id=None, cloud_id=None, notification_channel=None, local_vars_configuration=None):  # noqa: E501
        """CreateInstanceUsageBudget - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._budget_amount = None
        self._evaluation_period = None
        self._budget_unit = None
        self._project_id = None
        self._cloud_id = None
        self._notification_channel = None
        self.discriminator = None

        self.name = name
        self.budget_amount = budget_amount
        self.evaluation_period = evaluation_period
        if budget_unit is not None:
            self.budget_unit = budget_unit
        if project_id is not None:
            self.project_id = project_id
        if cloud_id is not None:
            self.cloud_id = cloud_id
        if notification_channel is not None:
            self.notification_channel = notification_channel

    @property
    def name(self):
        """Gets the name of this CreateInstanceUsageBudget.  # noqa: E501


        :return: The name of this CreateInstanceUsageBudget.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this CreateInstanceUsageBudget.


        :param name: The name of this CreateInstanceUsageBudget.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def budget_amount(self):
        """Gets the budget_amount of this CreateInstanceUsageBudget.  # noqa: E501


        :return: The budget_amount of this CreateInstanceUsageBudget.  # noqa: E501
        :rtype: float
        """
        return self._budget_amount

    @budget_amount.setter
    def budget_amount(self, budget_amount):
        """Sets the budget_amount of this CreateInstanceUsageBudget.


        :param budget_amount: The budget_amount of this CreateInstanceUsageBudget.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and budget_amount is None:  # noqa: E501
            raise ValueError("Invalid value for `budget_amount`, must not be `None`")  # noqa: E501

        self._budget_amount = budget_amount

    @property
    def evaluation_period(self):
        """Gets the evaluation_period of this CreateInstanceUsageBudget.  # noqa: E501


        :return: The evaluation_period of this CreateInstanceUsageBudget.  # noqa: E501
        :rtype: InstanceUsageBudgetEvaluationPeriod
        """
        return self._evaluation_period

    @evaluation_period.setter
    def evaluation_period(self, evaluation_period):
        """Sets the evaluation_period of this CreateInstanceUsageBudget.


        :param evaluation_period: The evaluation_period of this CreateInstanceUsageBudget.  # noqa: E501
        :type: InstanceUsageBudgetEvaluationPeriod
        """
        if self.local_vars_configuration.client_side_validation and evaluation_period is None:  # noqa: E501
            raise ValueError("Invalid value for `evaluation_period`, must not be `None`")  # noqa: E501

        self._evaluation_period = evaluation_period

    @property
    def budget_unit(self):
        """Gets the budget_unit of this CreateInstanceUsageBudget.  # noqa: E501


        :return: The budget_unit of this CreateInstanceUsageBudget.  # noqa: E501
        :rtype: InstanceUsageBudgetUnit
        """
        return self._budget_unit

    @budget_unit.setter
    def budget_unit(self, budget_unit):
        """Sets the budget_unit of this CreateInstanceUsageBudget.


        :param budget_unit: The budget_unit of this CreateInstanceUsageBudget.  # noqa: E501
        :type: InstanceUsageBudgetUnit
        """

        self._budget_unit = budget_unit

    @property
    def project_id(self):
        """Gets the project_id of this CreateInstanceUsageBudget.  # noqa: E501


        :return: The project_id of this CreateInstanceUsageBudget.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this CreateInstanceUsageBudget.


        :param project_id: The project_id of this CreateInstanceUsageBudget.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def cloud_id(self):
        """Gets the cloud_id of this CreateInstanceUsageBudget.  # noqa: E501


        :return: The cloud_id of this CreateInstanceUsageBudget.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this CreateInstanceUsageBudget.


        :param cloud_id: The cloud_id of this CreateInstanceUsageBudget.  # noqa: E501
        :type: str
        """

        self._cloud_id = cloud_id

    @property
    def notification_channel(self):
        """Gets the notification_channel of this CreateInstanceUsageBudget.  # noqa: E501


        :return: The notification_channel of this CreateInstanceUsageBudget.  # noqa: E501
        :rtype: CreateNotificationChannelRecord
        """
        return self._notification_channel

    @notification_channel.setter
    def notification_channel(self, notification_channel):
        """Sets the notification_channel of this CreateInstanceUsageBudget.


        :param notification_channel: The notification_channel of this CreateInstanceUsageBudget.  # noqa: E501
        :type: CreateNotificationChannelRecord
        """

        self._notification_channel = notification_channel

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateInstanceUsageBudget):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CreateInstanceUsageBudget):
            return True

        return self.to_dict() != other.to_dict()



import contextlib
import json
import os
from typing import Optional

import click
import tabulate

import anyscale
from anyscale.cli_logger import BlockLogger
from anyscale.cloud_utils import get_cloud_id_and_name
from anyscale.controllers.base_controller import BaseController
import anyscale.project_utils
from anyscale.project_utils import (  # pylint:disable=private-import
    attach_to_project_with_id,
    create_new_proj_def,
    get_proj_id_from_name,
    load_project_or_throw,
    register_or_attach_to_project,
    write_project_file_to_disk,
)
from anyscale.sdk.anyscale_client import (
    PageQuery,
    Project,
    ProjectListResponse,
    ProjectsQuery,
    TextQuery,
)
import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
from anyscale.util import get_endpoint


COMPUTE_CONFIG_FILENAME = "example_compute_config.json"


class ProjectController(BaseController):
    def __init__(
        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
    ):
        if log is None:
            log = BlockLogger()

        super().__init__(initialize_auth_api_client=initialize_auth_api_client)

        self.log = log
        self.log.open_block("Output")

    def get_proj_id_from_name(
        self, project_name: str, owner: Optional[str] = None
    ) -> str:
        """ Call API to get project id given a project name.  """
        return get_proj_id_from_name(project_name, self.api_client, owner)

    def clone(self, project_name: str, owner: Optional[str] = None) -> None:
        project_id = get_proj_id_from_name(project_name, self.api_client, owner)

        os.makedirs(project_name)
        write_project_file_to_disk(project_id, project_name)

        self._write_sample_compute_config(
            filepath=os.path.join(project_name, COMPUTE_CONFIG_FILENAME),
            project_id=project_id,
        )

    def create(self, name: str, parent_cloud_id: Optional[str]) -> None:
        """
        Call API to create a new project given a project name.
        """
        if not parent_cloud_id:
            raise click.ClickException(
                "Parent cloud id should be specified when creating a new project because "
                "cloud isolation has been enabled."
            )

        # Create new project
        project_id = self.api_client.create_project_api_v2_projects_post(
            write_project={
                "name": name,
                "description": "",
                "initial_cluster_config": None,
                "parent_cloud_id": parent_cloud_id,
            }
        ).result.id

        self.log.info(f"Created project {project_id} with name {name}.")

    def init(
        self,
        project_id: Optional[str],
        name: Optional[str],
        config: Optional[str] = None,
        requirements: Optional[str] = None,  # noqa: ARG002
    ) -> None:
        if config:
            message = (
                "Warning: `anyscale init` is no longer accepting a cluster yaml in "
                "the --config argument. All sessions should be started with cluster envs "
                "and compute configs."
            )
            self.log.warning(message)
        project_id_path = anyscale.project_utils.ANYSCALE_PROJECT_FILE

        if project_id:
            # Exactly one of project_id or name must be provided
            assert not name
            attach_to_project_with_id(project_id, self.api_client)
        else:
            # Exactly one of project_id or name must be provided
            assert name

            if os.path.exists(project_id_path):
                # Project id exists.
                project_definition = load_project_or_throw()
                project_id = project_definition.config["project_id"]

                # Checking if the project is already registered.
                # TODO: Fetch project by id rather than listing all projects
                try:
                    resp = self.api_client.get_project_api_v2_projects_project_id_get(
                        project_id
                    )
                except Exception:  # noqa: BLE001
                    resp = None
                if resp and resp.result.id == project_id:
                    if name != resp.result.name:
                        self.log.info(
                            "This directory is already attached to a project named {name}: {url}. ".format(
                                name=resp.result.name,
                                url=get_endpoint(f"/projects/{resp.result.id}"),
                            )
                            + "Delete .anyscale.yaml if you wish to attach to a different project."
                        )
                    else:
                        self.log.info(
                            "This directory is already attached to this project: {url}".format(
                                url=get_endpoint(f"/projects/{resp.result.id}")
                            )
                        )
                    return
                # Project id exists locally but not registered in the db.
                if click.confirm(
                    "The Anyscale project associated with this directory doesn't "
                    "seem to exist anymore or is not valid. Do you want "
                    "to re-init this directory?",
                    abort=True,
                ):
                    os.remove(project_id_path)
                    _, project_definition = create_new_proj_def(
                        name, api_client=self.api_client,
                    )
            else:
                # Project id doesn't exist and not enough info to create project.
                _, project_definition = create_new_proj_def(
                    name, api_client=self.api_client,
                )

            register_or_attach_to_project(project_definition, self.api_client)

    def _write_sample_compute_config(
        self, filepath: str, project_id: Optional[str] = None,
    ) -> None:
        """Writes a sample compute config JSON file to be used with anyscale up.
        If no default cloud is available from the organization and the user
        has never used a cloud before, don't write the sample compute config.
        """

        # Compute configs need a real cloud ID.
        cloud_id = None
        user = self.api_client.get_user_info_api_v2_userinfo_get().result
        organization = user.organizations[0]  # Each user only has one org
        if organization.default_cloud_id:
            # Use default cloud id if organization has one and if user has correct
            # permissions for it.
            with contextlib.suppress(Exception):
                get_cloud_id_and_name(
                    self.api_client, cloud_id=organization.default_cloud_id
                )
                cloud_id = organization.default_cloud_id
        if not cloud_id and project_id:
            # See if the project has a cloud ID for us to use.
            project = self.anyscale_api_client.get_project(project_id).result
            cloud_id = project.last_used_cloud_id

        # If no cloud ID in the project, fall back to the oldest cloud.
        # (For full compatibility with other frontends, we should be getting
        # the last used cloud from the user, but our users APIs
        # are far from in good shape for that...)
        if not cloud_id:
            cloud_list = self.api_client.list_clouds_api_v2_clouds_get().results
            if len(cloud_list) == 0:
                # If there is no cloud ID from the user,
                # let them create their project and set up a
                # compute config JSON file later.
                return
            cloud_id = cloud_list[-1].id

        default_config = self.api_client.get_default_compute_config_api_v2_compute_templates_default_cloud_id_get(
            cloud_id=cloud_id
        ).result

        with open(filepath, "w") as f:
            json.dump(default_config.to_dict(), f, indent=2)

    def list(self, name: str, json_format: bool, created_by_user: bool, max_items: int):
        paging_count = 20
        page_query = PageQuery(count=paging_count)
        name_query = None
        creator_query = None

        if created_by_user:
            user_info_response = self.api_client.get_user_info_api_v2_userinfo_get()
            user_id = user_info_response.result.id
            creator_query = TextQuery(equals=user_id)

        if name:
            name_query = TextQuery(equals=name)

        response = self._make_projects_query(name_query, creator_query, page_query)
        prepared_result = self._format_project_results(response, json_format)
        next_paging_token = response.metadata.next_paging_token
        has_more = (next_paging_token is not None) and (
            len(prepared_result) < max_items
        )
        while has_more:
            page_query = PageQuery(count=paging_count, paging_token=next_paging_token)
            response = self._make_projects_query(name_query, creator_query, page_query)
            prepared_result = prepared_result + self._format_project_results(
                response, json_format
            )
            next_paging_token = response.metadata.next_paging_token
            has_more = (next_paging_token is not None) and (
                len(prepared_result) < max_items
            )
        prepared_result = prepared_result[:max_items]

        if json_format:
            print(json.dumps(prepared_result))
        else:
            table = tabulate.tabulate(
                prepared_result,
                headers=["NAME", "ID", "URL", "DESCRIPTION"],
                tablefmt="plain",
            )
            print(f"Projects:\n{table}")

    def _format_project_dict(self, project: Project):
        return {
            "name": project.name,
            "id": project.id,
            "url": get_endpoint(f"/projects/{project.id}"),
            "description": project.description,
        }

    def _format_project_list(self, project: Project):
        return [
            project.name,
            project.id,
            f"{shared_anyscale_conf.ANYSCALE_HOST}/projects/{project.id}",
            project.description,
        ]

    def _format_project_results(self, response: ProjectListResponse, json_format: bool):
        if json_format:
            return [self._format_project_dict(result) for result in response.results]
        else:
            return [self._format_project_list(result) for result in response.results]

    def _make_projects_query(
        self,
        name_query: Optional[TextQuery],
        creator_query: Optional[TextQuery],
        page_query: Optional[PageQuery],
    ):
        project_query = ProjectsQuery(
            name=name_query, creator_id=creator_query, paging=page_query
        )
        return self.anyscale_api_client.search_projects(project_query)



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class MachinePoolSearchResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'machine_pool_name': 'str',
        'machine_pool_id': 'str',
        'organization_id': 'str',
        'cloud_ids': 'list[str]',
        'cloud_resource_ids': 'dict(str, list[str])',
        'spec': 'object'
    }

    attribute_map = {
        'machine_pool_name': 'machine_pool_name',
        'machine_pool_id': 'machine_pool_id',
        'organization_id': 'organization_id',
        'cloud_ids': 'cloud_ids',
        'cloud_resource_ids': 'cloud_resource_ids',
        'spec': 'spec'
    }

    def __init__(self, machine_pool_name=None, machine_pool_id=None, organization_id=None, cloud_ids=None, cloud_resource_ids=None, spec=None, local_vars_configuration=None):  # noqa: E501
        """MachinePoolSearchResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._machine_pool_name = None
        self._machine_pool_id = None
        self._organization_id = None
        self._cloud_ids = None
        self._cloud_resource_ids = None
        self._spec = None
        self.discriminator = None

        self.machine_pool_name = machine_pool_name
        self.machine_pool_id = machine_pool_id
        self.organization_id = organization_id
        self.cloud_ids = cloud_ids
        if cloud_resource_ids is not None:
            self.cloud_resource_ids = cloud_resource_ids
        if spec is not None:
            self.spec = spec

    @property
    def machine_pool_name(self):
        """Gets the machine_pool_name of this MachinePoolSearchResult.  # noqa: E501

        The name of the machine pool.  # noqa: E501

        :return: The machine_pool_name of this MachinePoolSearchResult.  # noqa: E501
        :rtype: str
        """
        return self._machine_pool_name

    @machine_pool_name.setter
    def machine_pool_name(self, machine_pool_name):
        """Sets the machine_pool_name of this MachinePoolSearchResult.

        The name of the machine pool.  # noqa: E501

        :param machine_pool_name: The machine_pool_name of this MachinePoolSearchResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and machine_pool_name is None:  # noqa: E501
            raise ValueError("Invalid value for `machine_pool_name`, must not be `None`")  # noqa: E501

        self._machine_pool_name = machine_pool_name

    @property
    def machine_pool_id(self):
        """Gets the machine_pool_id of this MachinePoolSearchResult.  # noqa: E501

        The unique identifier of the machine pool.  # noqa: E501

        :return: The machine_pool_id of this MachinePoolSearchResult.  # noqa: E501
        :rtype: str
        """
        return self._machine_pool_id

    @machine_pool_id.setter
    def machine_pool_id(self, machine_pool_id):
        """Sets the machine_pool_id of this MachinePoolSearchResult.

        The unique identifier of the machine pool.  # noqa: E501

        :param machine_pool_id: The machine_pool_id of this MachinePoolSearchResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and machine_pool_id is None:  # noqa: E501
            raise ValueError("Invalid value for `machine_pool_id`, must not be `None`")  # noqa: E501

        self._machine_pool_id = machine_pool_id

    @property
    def organization_id(self):
        """Gets the organization_id of this MachinePoolSearchResult.  # noqa: E501

        The organization ID that owns the machine pool.  # noqa: E501

        :return: The organization_id of this MachinePoolSearchResult.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this MachinePoolSearchResult.

        The organization ID that owns the machine pool.  # noqa: E501

        :param organization_id: The organization_id of this MachinePoolSearchResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def cloud_ids(self):
        """Gets the cloud_ids of this MachinePoolSearchResult.  # noqa: E501

        The cloud IDs that are attached to the machine pool.  # noqa: E501

        :return: The cloud_ids of this MachinePoolSearchResult.  # noqa: E501
        :rtype: list[str]
        """
        return self._cloud_ids

    @cloud_ids.setter
    def cloud_ids(self, cloud_ids):
        """Sets the cloud_ids of this MachinePoolSearchResult.

        The cloud IDs that are attached to the machine pool.  # noqa: E501

        :param cloud_ids: The cloud_ids of this MachinePoolSearchResult.  # noqa: E501
        :type: list[str]
        """
        if self.local_vars_configuration.client_side_validation and cloud_ids is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_ids`, must not be `None`")  # noqa: E501

        self._cloud_ids = cloud_ids

    @property
    def cloud_resource_ids(self):
        """Gets the cloud_resource_ids of this MachinePoolSearchResult.  # noqa: E501

        The cloud resource IDs for each cloud ID that is attached to the machine pool.  # noqa: E501

        :return: The cloud_resource_ids of this MachinePoolSearchResult.  # noqa: E501
        :rtype: dict(str, list[str])
        """
        return self._cloud_resource_ids

    @cloud_resource_ids.setter
    def cloud_resource_ids(self, cloud_resource_ids):
        """Sets the cloud_resource_ids of this MachinePoolSearchResult.

        The cloud resource IDs for each cloud ID that is attached to the machine pool.  # noqa: E501

        :param cloud_resource_ids: The cloud_resource_ids of this MachinePoolSearchResult.  # noqa: E501
        :type: dict(str, list[str])
        """

        self._cloud_resource_ids = cloud_resource_ids

    @property
    def spec(self):
        """Gets the spec of this MachinePoolSearchResult.  # noqa: E501

        A machine pool specification; schema and allowed fields are defined in Anyscale documentation.  # noqa: E501

        :return: The spec of this MachinePoolSearchResult.  # noqa: E501
        :rtype: object
        """
        return self._spec

    @spec.setter
    def spec(self, spec):
        """Sets the spec of this MachinePoolSearchResult.

        A machine pool specification; schema and allowed fields are defined in Anyscale documentation.  # noqa: E501

        :param spec: The spec of this MachinePoolSearchResult.  # noqa: E501
        :type: object
        """

        self._spec = spec

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MachinePoolSearchResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MachinePoolSearchResult):
            return True

        return self.to_dict() != other.to_dict()



from typing import List, Optional

from anyscale._private.sdk.base_sdk import BaseSDK
from anyscale.client.openapi_client.models.create_resource_quota import (
    CreateResourceQuota as CreateResourceQuotaModel,
)
from anyscale.client.openapi_client.models.quota import Quota as QuotaModel
from anyscale.resource_quota.models import CreateResourceQuota, Quota, ResourceQuota


class PrivateResourceQuotaSDK(BaseSDK):
    def create(self, create_resource_quota: CreateResourceQuota) -> ResourceQuota:
        cloud_id = self.client.get_cloud_id(
            cloud_name=create_resource_quota.cloud, compute_config_id=None
        )

        project_id = (
            self.client.get_project_id(
                parent_cloud_id=cloud_id, name=create_resource_quota.project
            )
            if create_resource_quota.project
            else None
        )

        user_id = None
        if create_resource_quota.user_email:
            users = self.client.get_organization_collaborators(
                email=create_resource_quota.user_email
            )

            if len(users) == 0:
                raise ValueError(
                    f"User with email '{create_resource_quota.user_email}' not found."
                )

            if len(users) > 1:
                raise ValueError(
                    f"Multiple users found for email '{create_resource_quota.user_email}'. Please contact Anyscale support."
                )
            user_id = users[0].user_id

        create_resource_quota_model = CreateResourceQuotaModel(
            name=create_resource_quota.name,
            cloud_id=cloud_id,
            project_id=project_id,
            user_id=user_id,
            quota=QuotaModel(
                num_cpus=create_resource_quota.num_cpus,
                num_instances=create_resource_quota.num_instances,
                num_gpus=create_resource_quota.num_gpus,
                num_accelerators=create_resource_quota.num_accelerators,
            ),
        )

        resource_quota = self.client.create_resource_quota(
            create_resource_quota=create_resource_quota_model
        )

        return ResourceQuota(
            id=resource_quota.id,
            name=resource_quota.name,
            cloud_id=resource_quota.cloud_id,
            project_id=resource_quota.project_id,
            user_id=resource_quota.user_id,
            is_enabled=resource_quota.is_enabled,
            created_at=resource_quota.created_at,
            deleted_at=resource_quota.deleted_at,
            quota=Quota(
                num_cpus=resource_quota.quota.num_cpus,
                num_instances=resource_quota.quota.num_instances,
                num_gpus=resource_quota.quota.num_gpus,
                num_accelerators=resource_quota.quota.num_accelerators,
            ),
        )

    def list(
        self,
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        creator_id: Optional[str] = None,
        is_enabled: Optional[bool] = None,
        max_items: int = 20,
    ) -> List[ResourceQuota]:
        cloud_id = (
            self.client.get_cloud_id(cloud_name=cloud, compute_config_id=None)
            if cloud
            else None
        )

        resource_quotas = self.client.list_resource_quotas(
            name, cloud_id, creator_id, is_enabled, max_items,
        )

        return [
            ResourceQuota(
                id=resource_quota.id,
                name=resource_quota.name,
                cloud_id=resource_quota.cloud_id,
                project_id=resource_quota.project_id,
                user_id=resource_quota.user_id,
                is_enabled=resource_quota.is_enabled,
                created_at=resource_quota.created_at,
                deleted_at=resource_quota.deleted_at,
                quota=Quota(
                    num_cpus=resource_quota.quota.num_cpus,
                    num_instances=resource_quota.quota.num_instances,
                    num_gpus=resource_quota.quota.num_gpus,
                    num_accelerators=resource_quota.quota.num_accelerators,
                ),
            )
            for resource_quota in resource_quotas
        ]

    def delete(self, resource_quota_id: str) -> None:
        self.client.delete_resource_quota(resource_quota_id=resource_quota_id)

    def set_status(self, resource_quota_id: str, is_enabled: bool) -> None:
        self.client.set_resource_quota_status(
            resource_quota_id=resource_quota_id, is_enabled=is_enabled
        )



from typing import List, Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.service_account._private.service_account_sdk import (
    PrivateServiceAccountSDK,
)
from anyscale.service_account.commands import (
    _CREATE_API_KEY_DOCSTRINGS,
    _CREATE_API_KEY_EXAMPLE,
    _CREATE_DOCSTRINGS,
    _CREATE_EXAMPLE,
    _DELETE_DOCSTRINGS,
    _DELETE_EXAMPLE,
    _LIST_DOCSTRINGS,
    _LIST_EXAMPLE,
    _ROTATE_API_KEYS_DOCSTRINGS,
    _ROTATE_API_KEYS_EXAMPLE,
    create as create,
    create_api_key as create_api_key,
    delete as delete,
    list as list,  # noqa: A004 - claude_comment("claude-opus-4-5", "SDK public API re-export")
    rotate_api_keys as rotate_api_keys,
)
from anyscale.service_account.models import ServiceAccount


class ServiceAccountSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateServiceAccountSDK(
            client=client, logger=logger, timer=timer
        )

    @sdk_docs(
        doc_py_example=_CREATE_EXAMPLE, arg_docstrings=_CREATE_DOCSTRINGS,
    )
    def create(self, name: str) -> str:  # noqa: F811
        """Create a service account and return the API key.
        """
        return self._private_sdk.create(name)

    @sdk_docs(
        doc_py_example=_CREATE_API_KEY_EXAMPLE,
        arg_docstrings=_CREATE_API_KEY_DOCSTRINGS,
    )
    def create_api_key(  # noqa: F811
        self, email: Optional[str] = None, name: Optional[str] = None
    ) -> str:
        """Create an API key for the service account and return the API key.
        """
        return self._private_sdk.create_api_key(email, name)

    @sdk_docs(
        doc_py_example=_LIST_EXAMPLE, arg_docstrings=_LIST_DOCSTRINGS,
    )
    def list(self, max_items: int = 20,) -> List[ServiceAccount]:  # noqa: F811
        """List service accounts.
        """
        return self._private_sdk.list(max_items=max_items,)

    @sdk_docs(
        doc_py_example=_DELETE_EXAMPLE, arg_docstrings=_DELETE_DOCSTRINGS,
    )
    def delete(  # noqa: F811
        self, email: Optional[str] = None, name: Optional[str] = None
    ):
        """Delete a service account.
        """
        return self._private_sdk.delete(email, name)

    @sdk_docs(
        doc_py_example=_ROTATE_API_KEYS_EXAMPLE,
        arg_docstrings=_ROTATE_API_KEYS_DOCSTRINGS,
    )
    def rotate_api_key(
        self, email: Optional[str] = None, name: Optional[str] = None
    ) -> str:  # noqa: F811
        """Rotate all api keys of a service account and return the new API key.
        """
        return self._private_sdk.rotate_api_keys(email, name)



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class OperatorStatusDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'operator_version': 'str',
        'check_results': 'list[OperatorCheckResult]',
        'reported_at': 'datetime'
    }

    attribute_map = {
        'operator_version': 'operator_version',
        'check_results': 'check_results',
        'reported_at': 'reported_at'
    }

    def __init__(self, operator_version=None, check_results=None, reported_at=None, local_vars_configuration=None):  # noqa: E501
        """OperatorStatusDetails - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._operator_version = None
        self._check_results = None
        self._reported_at = None
        self.discriminator = None

        if operator_version is not None:
            self.operator_version = operator_version
        if check_results is not None:
            self.check_results = check_results
        if reported_at is not None:
            self.reported_at = reported_at

    @property
    def operator_version(self):
        """Gets the operator_version of this OperatorStatusDetails.  # noqa: E501

        The version of the Anyscale Operator.  # noqa: E501

        :return: The operator_version of this OperatorStatusDetails.  # noqa: E501
        :rtype: str
        """
        return self._operator_version

    @operator_version.setter
    def operator_version(self, operator_version):
        """Sets the operator_version of this OperatorStatusDetails.

        The version of the Anyscale Operator.  # noqa: E501

        :param operator_version: The operator_version of this OperatorStatusDetails.  # noqa: E501
        :type: str
        """

        self._operator_version = operator_version

    @property
    def check_results(self):
        """Gets the check_results of this OperatorStatusDetails.  # noqa: E501

        The results of the operator checks.  # noqa: E501

        :return: The check_results of this OperatorStatusDetails.  # noqa: E501
        :rtype: list[OperatorCheckResult]
        """
        return self._check_results

    @check_results.setter
    def check_results(self, check_results):
        """Sets the check_results of this OperatorStatusDetails.

        The results of the operator checks.  # noqa: E501

        :param check_results: The check_results of this OperatorStatusDetails.  # noqa: E501
        :type: list[OperatorCheckResult]
        """

        self._check_results = check_results

    @property
    def reported_at(self):
        """Gets the reported_at of this OperatorStatusDetails.  # noqa: E501

        The timestamp when the operator status was last reported.  # noqa: E501

        :return: The reported_at of this OperatorStatusDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._reported_at

    @reported_at.setter
    def reported_at(self, reported_at):
        """Sets the reported_at of this OperatorStatusDetails.

        The timestamp when the operator status was last reported.  # noqa: E501

        :param reported_at: The reported_at of this OperatorStatusDetails.  # noqa: E501
        :type: datetime
        """

        self._reported_at = reported_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OperatorStatusDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OperatorStatusDetails):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class FileStorage(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'file_storage_id': 'str',
        'mount_targets': 'list[NFSMountTarget]',
        'mount_path': 'str',
        'persistent_volume_claim': 'str',
        'csi_ephemeral_volume_driver': 'str'
    }

    attribute_map = {
        'file_storage_id': 'file_storage_id',
        'mount_targets': 'mount_targets',
        'mount_path': 'mount_path',
        'persistent_volume_claim': 'persistent_volume_claim',
        'csi_ephemeral_volume_driver': 'csi_ephemeral_volume_driver'
    }

    def __init__(self, file_storage_id=None, mount_targets=None, mount_path=None, persistent_volume_claim=None, csi_ephemeral_volume_driver=None, local_vars_configuration=None):  # noqa: E501
        """FileStorage - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._file_storage_id = None
        self._mount_targets = None
        self._mount_path = None
        self._persistent_volume_claim = None
        self._csi_ephemeral_volume_driver = None
        self.discriminator = None

        if file_storage_id is not None:
            self.file_storage_id = file_storage_id
        if mount_targets is not None:
            self.mount_targets = mount_targets
        if mount_path is not None:
            self.mount_path = mount_path
        if persistent_volume_claim is not None:
            self.persistent_volume_claim = persistent_volume_claim
        if csi_ephemeral_volume_driver is not None:
            self.csi_ephemeral_volume_driver = csi_ephemeral_volume_driver

    @property
    def file_storage_id(self):
        """Gets the file_storage_id of this FileStorage.  # noqa: E501

        For AWS, the EFS ID. For GCP, the Filestore instance name.  # noqa: E501

        :return: The file_storage_id of this FileStorage.  # noqa: E501
        :rtype: str
        """
        return self._file_storage_id

    @file_storage_id.setter
    def file_storage_id(self, file_storage_id):
        """Sets the file_storage_id of this FileStorage.

        For AWS, the EFS ID. For GCP, the Filestore instance name.  # noqa: E501

        :param file_storage_id: The file_storage_id of this FileStorage.  # noqa: E501
        :type: str
        """

        self._file_storage_id = file_storage_id

    @property
    def mount_targets(self):
        """Gets the mount_targets of this FileStorage.  # noqa: E501

        The mount target(s) to use.  # noqa: E501

        :return: The mount_targets of this FileStorage.  # noqa: E501
        :rtype: list[NFSMountTarget]
        """
        return self._mount_targets

    @mount_targets.setter
    def mount_targets(self, mount_targets):
        """Sets the mount_targets of this FileStorage.

        The mount target(s) to use.  # noqa: E501

        :param mount_targets: The mount_targets of this FileStorage.  # noqa: E501
        :type: list[NFSMountTarget]
        """

        self._mount_targets = mount_targets

    @property
    def mount_path(self):
        """Gets the mount_path of this FileStorage.  # noqa: E501

        For GCP, the Filestore root directory. For NFS, the path of the server to mount from (e.g., <mount-target-address>/<mount-path> will be mounted).  # noqa: E501

        :return: The mount_path of this FileStorage.  # noqa: E501
        :rtype: str
        """
        return self._mount_path

    @mount_path.setter
    def mount_path(self, mount_path):
        """Sets the mount_path of this FileStorage.

        For GCP, the Filestore root directory. For NFS, the path of the server to mount from (e.g., <mount-target-address>/<mount-path> will be mounted).  # noqa: E501

        :param mount_path: The mount_path of this FileStorage.  # noqa: E501
        :type: str
        """

        self._mount_path = mount_path

    @property
    def persistent_volume_claim(self):
        """Gets the persistent_volume_claim of this FileStorage.  # noqa: E501

        For Kubernetes resources, the name of the persistent volume claim used to mount shared storage into pods.  # noqa: E501

        :return: The persistent_volume_claim of this FileStorage.  # noqa: E501
        :rtype: str
        """
        return self._persistent_volume_claim

    @persistent_volume_claim.setter
    def persistent_volume_claim(self, persistent_volume_claim):
        """Sets the persistent_volume_claim of this FileStorage.

        For Kubernetes resources, the name of the persistent volume claim used to mount shared storage into pods.  # noqa: E501

        :param persistent_volume_claim: The persistent_volume_claim of this FileStorage.  # noqa: E501
        :type: str
        """

        self._persistent_volume_claim = persistent_volume_claim

    @property
    def csi_ephemeral_volume_driver(self):
        """Gets the csi_ephemeral_volume_driver of this FileStorage.  # noqa: E501

        For Kubernetes resources, the CSI ephemeral volume driver used to mount shared storage into pods.  # noqa: E501

        :return: The csi_ephemeral_volume_driver of this FileStorage.  # noqa: E501
        :rtype: str
        """
        return self._csi_ephemeral_volume_driver

    @csi_ephemeral_volume_driver.setter
    def csi_ephemeral_volume_driver(self, csi_ephemeral_volume_driver):
        """Sets the csi_ephemeral_volume_driver of this FileStorage.

        For Kubernetes resources, the CSI ephemeral volume driver used to mount shared storage into pods.  # noqa: E501

        :param csi_ephemeral_volume_driver: The csi_ephemeral_volume_driver of this FileStorage.  # noqa: E501
        :type: str
        """

        self._csi_ephemeral_volume_driver = csi_ephemeral_volume_driver

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FileStorage):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, FileStorage):
            return True

        return self.to_dict() != other.to_dict()



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from openapi_client.api.customer_offboarding_api import CustomerOffboardingApi
from openapi_client.api.default_api import DefaultApi



"""
Functionality for formatting output strings for Clouds
"""
import json
from typing import List

import tabulate

from anyscale.client.openapi_client.models.cloud import Cloud


def format_clouds_output(clouds: List[Cloud], json_format: bool) -> str:
    return (
        format_clouds_output_json(clouds=clouds)
        if json_format
        else format_clouds_output_table(clouds=clouds)
    )


def format_clouds_output_json(clouds: List[Cloud]) -> str:
    cloud_jsons = []
    for cloud in clouds:
        # Check if log ingestion is enabled based on is_aggregated_logs_enabled field
        log_ingestion_enabled = getattr(cloud, "is_aggregated_logs_enabled", False)
        cloud_jsons.append(
            {
                "id": cloud.id,
                "name": cloud.name,
                "provider": cloud.provider,
                "region": cloud.region,
                "added_date": cloud.created_at.strftime("%m/%d/%Y"),
                "default": cloud.is_default,
                "credentials": cloud.credentials,
                "log_ingestion": "enabled" if log_ingestion_enabled else "disabled",
            }
        )
    return json.dumps(cloud_jsons)


def format_clouds_output_table(clouds: List[Cloud]) -> str:
    table_rows = []
    for cloud in clouds:
        # Check if log ingestion is enabled based on is_aggregated_logs_enabled field
        log_ingestion_enabled = getattr(cloud, "is_aggregated_logs_enabled", False)
        log_ingestion_status = "enabled" if log_ingestion_enabled else "disabled"
        table_rows.append(
            [
                cloud.id,
                cloud.name,
                cloud.provider,
                cloud.region,
                cloud.created_at.strftime("%m/%d/%Y"),
                cloud.is_default,
                cloud.credentials,
                log_ingestion_status,
            ]
        )
    table = tabulate.tabulate(
        table_rows,
        headers=[
            "ID",
            "NAME",
            "PROVIDER",
            "REGION",
            "ADDED DATE",
            "DEFAULT",
            "CREDENTIALS",
            "LOG INGESTION",
        ],
        tablefmt="plain",
    )

    return f"Clouds:\n{table}"
